

class ProjectNotFound(Exception):
    pass

class ProjectParserError(Exception):
    pass



class ToolchainError(Exception):
    pass


class InvalidTarget(Exception):
    pass




class InstallError(Exception):
    pass


class SerialCannotOpenException(Exception):
    pass


class ReadElfError(Exception):
    pass


class CodeSyncError(Exception):
    pass



class BranchError(Exception):
    pass


import time
import re
import sys
import io
import tempfile
import logging


import pexpect
from pexpect.spawnbase import SpawnBase


class SerialSpawn(SpawnBase):
    """Due to pyserial not support file descriptor for windows, fdspawn could not be used.
    This class implement a spawn for pyserial, all interfaces are defined from the base
    `SpawnBase`. For more information about the usage, please refer the pexpect documentation.

    Note: logfile_read detault is used in it's internal.

    Simple example:
        >>> ser = serial.Serial('COM3', 9600, timeout=0.1)
        >>> spawn = SerialSpawn(ser)
        >>> spawn.logfile = sys.stdout
        >>> spawn.write_expect("A", "Waiting for power mode select..", timeout=3)
        >>> print spawn.before
        >>> print spawn.after
        >>> spawn.expect([pexpect.EOF, pexpect.TIMEOUT], timeout=1)
        >>> spawn.close()
    """
    def __init__(self, serial, timeout=3, searchwindowsize=None,
            logfile=None, logfile_read=None, encoding=None, codec_errors='strict'):

        if not serial.is_open:
            serial.open()

        if hasattr(serial, 'reader_isalive'):
            if serial.reader_isalive:
                serial.stop_reader()
                logging.debug('reader thread is stopped!')

        self.serial = serial
        super(SerialSpawn, self).__init__(
            timeout,
            maxread=1,
            searchwindowsize=searchwindowsize,
            logfile=logfile,
            encoding=encoding,
            codec_errors=codec_errors
        )

        self.logfile_read = logfile_read
        self.closed = not self.serial.is_open


    def read_nonblocking(self, size=1, timeout=None):
        """This is fake nonblocking, the size is decided by how many data in the buffer, rather than
        specific value, this is because big size will block the serial read, small size will effect the
        performance when many data in buffer. timeout is useless.
        """
        time.sleep(0.1)
        s = self.serial.read(self.serial.in_waiting)
        s = self._decoder.decode(s, final=False)
        self._log(s, 'read')
        return s



    def send(self, s):
        """Send data to serial, and logging to log_send."""
        logging.info("SEND: %s", s)
        s = self._coerce_send_string(s)
        self._log(s, 'send')
        self._log("\n>>>%s\n"%s, 'read')
        b = self._encoder.encode(s, final=False)
        return self.serial.write(b)


    def sendline(self, s):
        """Send line"""
        s = self._coerce_send_string(s)
        return self.send(s + self.linesep)


    def write(self, s):
        self.send(s)


    def writelines(self, sequence):
        for s in sequence:
            self.write(s)



    def flush(self):
        self.serial.flush()


    def flush_log(self):
        if hasattr(self.serial, 'append_data') and isinstance(self.logfile_read, io.BytesIO):
            logging.debug("dump reading log to serial object!")
            self.logfile_read.seek(0)
            data = self.logfile_read.read()
            self.serial.append_data(data)



    def find(self, pattern, timeout=30):
        """Return the matches of pattern within a specific timeout
        in the serial reading stream. If EOF, return value is None.

        If timeout occured, that will raise pexpect.TIMEOUT exception.
        """
        try:
            self.expect(pattern, timeout=timeout)
            return self.before + self.after
        except pexpect.EOF:
            return None



    def test_input(self, input, expect, timeout=30):
        """Input value to serial, and test the output if match the expectation.

        Arguments:
            input {str} -- input string
            expect {str} -- pattern
            timeout {float} -- timeout in seconds

        Returns:
            str -- the output
        """
        self.write(input)
        self.expect(expect, timeout=timeout)
        self.flush_log()

        return self.before + self.after



    def close(self):
        """Close serial port, and dump the logfile_read to mcutk.pserial.Serial.data.
        If the serial instance is comes from pyserial, dump action will not take.
        """
        self.serial.close()
        self.flush_log()





    def isalive (self):
        return self.serial.is_open





# Copyright (c) 2015-2016 Freescale Semiconductor, Inc.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# o Redistributions of source code must retain the above copyright notice, this list
#   of conditions and the following disclaimer.
#
# o Redistributions in binary form must reproduce the above copyright notice, this
#   list of conditions and the following disclaimer in the documentation and/or
#   other materials provided with the distribution.
#
# o Neither the name of Freescale Semiconductor, Inc. nor the names of its
#   contributors may be used to endorse or promote products derived from this
#   software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
from __future__ import absolute_import
import os
import sys
import time
import logging
import tempfile
import subprocess
import threading
import importlib

from mcutk.debugger.base import DebuggerBase
from mcutk.pserial.serial import Serial


def getboard(name, **kwargs):
    """An entry to get board instance.

    Arguments:
        name {string} -- board name
    """
    devicename = kwargs.pop("devicename", "")
    try:
        boardmodule_path = "mcutk.board.%s"%name
        logging.debug(boardmodule_path)
        boardmodule = importlib.import_module(boardmodule_path)
        board = boardmodule.Board(devicename, **kwargs)
    except ImportError as e:
        board = Board(devicename, **kwargs)

    logging.debug(str(board))
    board.name = name
    return board




class Board(object):
    """MCUTK base board. Defined common interface & functions.
    This object can be used directly and provide general support for Kinetis series.
    """
    def __init__(self, devicename, **kwargs):
        """Create a mcutk.Board instance.

        Arguments:
            devicename {string} -- device name
            interface {string} -- SWD/JTAG

        Keyword Arguments:
            debugger_type {string} -- debugger type, choices are defined in
        """
        self.name = devicename
        self.devicename = devicename
        self._debugger = None
        self._serial_ports = list()

        self.interface = kwargs.get("interface", "SWD")
        self.debugger_type = kwargs.get("debugger_type", "jlink")

        # default gdbport is 3000
        self.gdbport = kwargs.get("gdbport", 3333)
        self.usbid = kwargs.get("usbid")
        self.serial = kwargs.get("serial", "")
        self.baudrate = kwargs.get("baudrate", "115200")
        self.start_address = kwargs.get("start_address", "0")

        self.sp = "(0x00000000)"
        self.pc = "(0x00000004)"
        self.resource = []


    def __repr__(self):
        return "{0}(name={1.devicename})".format(self.__class__, self)


    def set_serial(self, port, baudrate, **kwargs):
        """Set or add serial port to board object, this interface will pass all
        parameters to serial.Serial object. For more details, please refer to pyserial
        documentation: https://pythonhosted.org/pyserial/pyserial_api.html#classes.

        Default timeout=1.
        """
        if not port:
            return None
        timeout = kwargs.pop('timeout', 1)
        sp = Serial(timeout=timeout, **kwargs)
        sp.port = port
        sp.baudrate = baudrate
        self._serial_ports.append(sp)


    def get_serial(self, index=0):
        """Get serial port instance by index.
            0 -- main
            1 -- secondary
            2 -- third

        Arguments:
            index {int} -- the port index.

        Returns:
            pyserial, serila.Serial instance,
        """
        if not self._serial_ports:
            logging.debug('no serial ports are configured!')
            return None

        try:
            return self._serial_ports[index]
        except IndexError:
            return None


    @property
    def ser_main(self):
        """A shortcut attribute to access the main serial port object.
        """
        return self.get_serial(0)


    @property
    def ser_sec(self):
        """A shortcut attribute to access the secondary serial port object.
        """
        return self.get_serial(1)


    def remove_resource(self, res_inst):
        for res in self.resource:
            if id(res[1]) == id(res_inst):
                logging.warning("find resource for %s", id(res_inst))
                self.resource.remove(res)

        logging.warning("resource for %s not found", id(res_inst))
        return None


    def register_resource(self, res_inst, naming):
        """
        regist resources to board
        res_init: resource instance
        naming: name string of this resource
        """
        res = [naming, res_inst]

        self.resource.insert(-1, res)


    def find_resource_by_name(self, naming):
        """
        find a resource by name
        naming: the name of the resource
        return: the first match resource or None
        """
        for res in self.resource:
            if res[0] == naming:
                return res[1]

        logging.debug("resource for %s not found", naming)
        return None


    def find_resource_by_type(self, type_string):
        """
        find a resource by type
        type_string: the name of resource type(class)
        return: a list of matched resource, otherwise None
        """
        ret = []
        for res in self.resource:
            if type(res[1]).__name__ == type_string:
                logging.info("find resource for %s", type_string)
                ret.insert(-1, res[1])

        logging.info("resource for %s not found", type_string)
        return None


    @property
    def debugger(self):
        if not self._debugger:
            raise ValueError("debugger is not set!")
        self._debugger.set_board(self)
        return self._debugger



    @debugger.setter
    def debugger(self, value):
        if isinstance(value, DebuggerBase):
            self._debugger = value
        else:
            ValueError("This not a valid debugger object")




    @property
    def gdb_init_commands(self):
        """gdb.init is a string include gdb commands.

        It will be rendered before execute 'gdb -x gdb.init'.
        Default it is loaded from debugger.gdbinit_template.
        Overwrite this function can custom the commands.
        """
        return None



    def reset_board_by_send_break(self, serial=None):
        """CMSIS-DAP firmware allows the target to be reset by sending a break command
        over the serial port.
        Default use the main serial port.
        """
        if serial == None:
            serial = self.ser_main

        logging.info('reset board by sending break to port: %s', serial.port)
        _opened_by_me = False
        if not serial.is_open:
            _opened_by_me = True
            serial.open()

        try:
            serial.send_break()
        except:
            serial.break_condition = False

        # if port status is aligned with the origin.
        if _opened_by_me:
            serial.close()

        return True



    def reset(self, method="debugger"):
        """Reset board. There are several methos allow user to reset board.
        By default it is debugger method.

        Reset method list:
            - debugger: use debugger(JTAG) to reset board
            - serial: send break via serial port

        Keyword Arguments:
            method {str} -- [description] (default: {"debugger"})
        """
        if method == 'serial':
            return self.reset_board_by_send_break()

        elif method == "debugger":
            assert self.debugger
            return self.debugger.reset()

        else:
            raise ValueError('unknow reset method %s'%method)





    def programming(self, filename, **kwargs):
        """Auto program binary to board.

        For general situation, it is avaliable for most boards.
        It will choose gdb or general method by filename extension.

        params:
            filename: path to image file.
        """
        logging.info("debugger version %s", self.debugger.version)
        logging.info("programming %s", filename)

        if self.debugger_type in ("jlink", 'pyocd'):
            if filename.split(".")[-1] in ("bin"):
                return self.debugger.flash(filename, self.start_address)
            else:
                return self.debugger.gdb_program(filename, **kwargs)
        else:
            return self.debugger.flash(filename, **kwargs)





    def check_serial(self):
        """Check serial port.
        """
        status = "pass"
        try:
            self.ser_main.write_timeout = 2
            self.ser_main.open()
            self.ser_main.write("A\r\n")
        except Exception as e:
            status = str(e)
        finally:
            if self.ser_main and self.ser_main.is_open:
                self.ser_main.close()

        return status










import os
import re
import platform
import glob
from distutils.version import LooseVersion

from mcutk.apps.decorators import build
from mcutk.apps.idebase import IDEBase, BuildResult
from mcutk.apps.eclipse import generate_build_cmdline

class APP(IDEBase):
    """ARM Development Studio(DS5/DS).

    Create an instance:
        (1) search in windows registry:
            >>> ds5 = APP.get_latest()
            >>> print ds5.version

        (2) from api:
            >>> ds5 = APP("C:\Program Files\DS-5 v5.22.0", version="5.22.0")
            >>> print ds5.version
    """

    OSLIST = ["Windows", "Linux"]

    EXITCODE = {
        0: "PASS",
        1: "Errors"
    }


    def __init__(self, path, *args, **kwargs):
        super(APP, self).__init__("ds5", path, *args, **kwargs)
        self.builder = os.path.join(self._path, "bin", "eclipse")



    @build
    def build_project(self, project, target, logfile, **kwargs):
        """Return a string about the build command line.

        Arguments:
            project {ds5.Project} -- ds5 project object
            target {string} -- target name
            logfile {string} -- log file path
            workspace {string} -- ds5 build workspace

        Returns:
            string -- build commandline

        """
        workspace = kwargs.get('workspace')
        cmdline = generate_build_cmdline(self.builder, workspace, project.prjdir, project.name, target)
        if logfile:
            cmdline.append('>> "{}" 2>&1'.format(logfile))
        return " ".join(cmdline)



    @staticmethod
    def get_latest():
        """Search and return a latest ds5 instance in system

        Returns:
            <ds5.APP object>
        """
        path, version = get_latest()
        if path:
            return APP(path, version=version)
        else:
            return None




    @staticmethod
    def parse_build_result(exitcode, logfile):
        """Parse ds5 build result

        Arguments:
            exitcode {int} -- the exit code of ds5 process
            logfile {string} -- a file contains the output from ds5.

        Returns:
            <BuildResult object> -- the build result

        Notes:
        ds5 exit code
            noerror warnnings: 0
            erros: none-zero

        Parser the log file by search the keyword: "Total number of warnings: <>".
        """
        content = []
        result = BuildResult.Pass
        if 0 == exitcode:
            with open(logfile, "r") as f:
                content = f.read()
                if re.search(r"Warning:", content):
                    result = BuildResult.Warnings
        else:
            result = BuildResult.Errors
        return result



    @staticmethod
    def verify(path):
        """Verify the path is valid ARM DS installation."""
        return os.path.exists(path+"/bin/eclipse")



    @property
    def is_ready(self):
        return os.path.exists(self.builder)



    @staticmethod
    def default_install_path(osname):
        default_path_table = {
            "Windows": r"C:\Program Files",
            "Linux": r"/usr/local"
        }
        return default_path_table[osname]


def get_latest():
    """Get the latest ARM DS instance

    Returns:
        tuple -- (path, version)
    """
    osname = platform.system()
    ds5_tool_list = []
    try:
        default_root_path = APP.default_install_path(osname)
        for per_folder in glob.glob(default_root_path + '/DS-5**'):
            ds5_tool_list.append(os.path.join(default_root_path, per_folder))

    except Exception:
        return None,None


    ds5_versions = {}
    for per_ds5 in ds5_tool_list:
        revfile = os.path.join(per_ds5, "/sw/info/rev.txt")
        if os.path.exists(revfile):
            with open(revfile, "r") as f:
                content = f.read()
            match = re.search(r"\d+", content)
            if match:
                ds5_versions[match.group()] = per_ds5

    try:
        latest_v = sorted(ds5_versions.keys(), key=lambda v:LooseVersion(v))[-1]
        return ds5_versions[latest_v], latest_v
    except IndexError:
        pass

    return None, None





import os
import glob
from xml.etree import cElementTree as ET
from mcutk.apps.projectbase import ProjectBase


class Project(ProjectBase):
    """
    IAR project object

    This class could parser the settings in *.ewp & *.eww.
    """
    PROJECT_EXTENSION = '*.ewp'

    def __init__(self, *args, **kwargs):
        super(Project, self).__init__(*args, **kwargs)
        try:
            ewpfile = glob.glob(os.path.join(self.prjdir, "*.ewp").replace("\\", "/"))[0]
            self.ewp_file = ewpfile
            self.ewp_xml = ET.parse(ewpfile)
        except IndexError:
            raise IOError("not found file with extension '.ewp'")

        self.eww_files = glob.glob(os.path.join(self.prjdir, "*.eww").replace("\\", "/"))
        self._conf = self._get_all_configuration()
        self._targets = self._conf.keys()
        self._name = os.path.basename(self.ewp_file).split('.')[0]


    def _get_all_configuration(self):
        """Read all configuration from *.ewp file

        Raises:
            IOError -- if *.ewp is not exists, it will raise an IOError.

        Returns:
            dict -- targets configuration
        """
        targets = dict()
        for conf in self.ewp_xml.findall("configuration"):
            target_name = conf.find("name").text.strip()
            output_dir = conf.find("./settings[name='General']/data/option[name='ExePath']/state")\
                                .text.strip()
            output_name = conf.find("./settings[name='ILINK']/data/option[name='IlinkOutputFile']/state")\
                                .text.strip()
            targets[target_name] = output_dir.replace("$PROJ_DIR$/", "") + '/' + output_name

        return targets


    def get_deps(self):
        """Get project dependecies.

        Return a list of project directory.
        """
        deps = list()
        nodes = self.ewp_xml.findall("configuration/settings[name='ILINK']/data/option[name='IlinkRawBinaryFile']/state")
        for node in nodes:
            if node is not None and node.text:
                p = node.text.strip().replace("$PROJ_DIR$", self.prjdir)
                path = os.path.abspath(p)
                deps.append(path)
        return deps




    @property
    def name(self):
        """Return the application name

        Returns:
            string --- app name
        """
        return self._name






import os
import re
import platform
import subprocess
import logging
import threading
import time
import tempfile
import shlex
from threading import Thread, Timer
from copy import deepcopy
from mcutk.debugger.base import DebuggerBase
from mcutk.util import run_command
from mcutk.apps import appfactory


class IDE(DebuggerBase):
    """IDE debugger
    """

    # supported ides
    TOOLS = [
        'iar',
        'mdk',
        'mcux'
    ]

    def __init__(self, ides, **kwargs):
        super(IDE, self).__init__("ide", '', **kwargs)
        if not isinstance(ides, list):
            raise TypeError('apps must be a list')
        self._ides = ides
        self.template_root = ''
        self._force_ide = None


    @property
    def is_ready(self):
        for ide in self._ides:
            if ide.is_ready is False:
                return False
        return True


    def set_force_ide(self, name):
        if name in self.TOOLS:
            self._force_ide = name



    def flash(self, debugfile, idename='mcux', target='flexspi_nor_debug', board=None, template_root=None, **kwargs):
        convert_map = {
            'armgcc': 'mdk',
            'uv4': 'mdk'
        }

        if self._force_ide:
            idename = self._force_ide
            logging.info("mandatory debugger run with %s", idename)

        idename = convert_map.get(idename, idename)

        if template_root is None:
            template_root = self.template_root

        if board is None:
            board = self._board

        # assert template projects root
        if not os.path.exists(template_root):
            raise IOError("template project[%s] is not exists!"%template_root)


        #assert tool name
        if idename not in IDE.TOOLS:
            raise ValueError("IDE [{}] is unsupported for board programming!".format(idename))

        for app in self._ides:
            if app.name == idename:
                prjdir = os.path.join(template_root, app.name)

                # workaround for iar programming
                if idename == 'iar':
                    if ":" not in board.usbid:
                        logging.warning('There is no header infomation in usbid, but IAR-CSPY need that\n!')
                else:
                    # workaround to make sure the origin board object is not changed.
                    board = deepcopy(board)
                    board.usbid = board.usbid.split(':')[-1]

                self._call_registered_callback("before-load")
                kwargs['gdbpath'] = self.gdbpath
                logging.info("IDE name: %s, Version: %s", app.name, app.version)
                ret = app.programming(board, prjdir, target, debugfile, **kwargs)

                return ret
        else:
            raise ValueError('{} path is not exists or not set!'.format(idename))



    def erase(self, target='debug'):
        from mcutk.debugger.redlink import RedLink
        prjdir = os.path.join(self.template_root, 'mcux')
        for app in self._ides:
            if app.name == 'mcux':
                mcux = app
                break
        else:
            raise ValueError('MCUXpressoIDE path is not exists or not set!')

        redlink = RedLink(mcux.path + '/ide', version=self.version)
        redlink.gdbpath = self.gdbpath
        redlink.template_root = prjdir
        self._board.usbid = self._board.usbid.split(':')[-1]
        redlink.set_board(self._board)
        return redlink.erase()





    def reset(self):
        pass










from __future__ import print_function
import click
import glob
import os
from os.path import join
from mcutk.apps import appfactory
from mcutk.exceptions import ProjectNotFound
from mcutk.managers.conf_mgr import ConfMgr

from . import TOOLCHAINS




@click.command('config', short_help='configuration(\"~/.mcutk\") management')
@click.option('--show', is_flag=True, help='show configuration from \"~/.mcutk\"')
@click.option('--auto', is_flag=True, help='auto scan your system, then configure into \"~/.mcutk\"')
def cli(show, auto):
    """Configuration Management Command"""
    cfger = ConfMgr.load()

    if show:
        if cfger.is_empty:
            print("Need to initialize the mcutk")
            return
        print(cfger)

    if auto:
        print("scaning toolchains on your system ...\n")
        for toolname in TOOLCHAINS:
            tool = appfactory(toolname)
            app = tool.get_latest()
            if app and app.is_ready:
                print(' ----------------------------------------------------- ')
                print(' [x] {:10s} version={}, path={}'.format(app.name, app.version, app.path))
                cfger.set_app(app)
            else:
                print('Warning: failed to set %s'%toolname)
        cfger.save()

        print("\n\"{}\" have been updated successfully!".format(cfger.CONFIG_FILE))




import os
import re
import platform
import glob

from distutils.version import LooseVersion
from mcutk.apps.decorators import build
from mcutk.apps.idebase import IDEBase, BuildResult

class APP(IDEBase):
    """
    kinetis-design-studio Workbench interface implementation.
    This class inherit from IDEBase, and it has the ability to build project,
    and parse the build result.

    Create an instance:
        (1) search in windows registry:
            >>> kds = APP.get_latest()
            >>> print kds.version

        (2) from api
            >>> kds = APP("C:/Freescale/KDS_3.0.0", version="3.0.0")
            >>> print kds.version

    """

    OSLIST = ["Windows", "Linux", "Mac"]

    EXITCODE = {
        0:	"PASS",
        1:	"Errors"
    }

    def __init__(self, path, *args, **kwargs):
        super(APP, self).__init__("kds", path, *args, **kwargs)
        self.builder = os.path.join(self._path, "eclipse", "kinetis-design-studio")


    @build
    def build_project(self, project, target, logfile, *kwargs):
        """Return a string about the build command line.

        Arguments:
            project {kds.Project} -- kds project object
            target {string} -- target name
            logfile {string} -- log file path
            workspace {string} -- kds build workspace
        Returns:
            string -- build commandline

        KDS command line guide is quoted from KDS Workbench reference:

            To build the project from the command line, use the KDS Command Line Build Utility
            (kinetis-design-studio) located in the eclipse directory. As input you use the project
            file, and the invocation syntax is:

                kinetis-design-studio --launcher.suppressErrors -nosplash
                -application {org.eclipse.cdt.managedbuilder.core.headlessbuild|other applications}
                [-cleanBuild|-build] {project_name_reg_ex/config_name_reg_ex | all}
                [-import|-importAll] {[uri:/]/path/to/project}}
                -data {workspace path}

            ------------------------------------------------
            Parameter                   Description
            ------------------------------------------------
            -application applicationId  The application to run. Applications are declared by plug-ins supplying extensions to the org.eclipse.core.runtime.applications extension point. This argument is
                                        typically not needed. If specified, the value overrides the value supplied by the configuration. If not specified, the Eclipse Workbench is run.
            --launcher.suppressErrors   If specified the executable will not display any error or message dialogs. This is useful if the executable is being used in an unattended situation
            -nosplash                   Controls whether or not the splash screen is shown.
            -data workspacePath         The path of the workspace on which to run the Eclipse platform. The workspace location is also the default location for projects. Relative paths are interpreted.
                                        relative to the directory that Eclipse was started from.
            -nosplash                   Runs the platform without putting up the splash screen.
            -import                     {[uri:/]/path/to/project} -- Import projects under URI.
            -importAll                  {[uri:/]/path/to/projectTreeURI} -- Import all projects under URI.
            -build                      {project_name_reg_ex{/config_reg_ex} | all} -- Build projects.
            -cleanBuild                 {project_name_reg_ex{/config_reg_ex} | all} -- Clean and  build projects.
            -I                          {include_path} -- additional include_path to add to tools
            -include                    {include_file} -- additional include_file to pass to tools
            -D                          {prepoc_define} -- addition preprocessor defines to pass to the tools
            -E                          {var=value} -- replace/add value to environment variable when running all tools
            -Ea                         {var=value} -- append value to environment variable when running all tools
            -Ep                         {var=value} -- prepend value to environment variable when running all tools
            -Er                         {var} -- remove/unset the given environment variable
            -T                          {toolid} {optionid=value} -- replace a tool option value in each configuration built
            -Ta                         {toolid} {optionid=value} -- append to a tool option value in each configuration built
            -Tp                         {toolid} {optionid=value} -- prepend to a tool option value in each configuration built
            -Tr                         {toolid} {optionid=value} -- remove a tool option value in each configuration built

        """

        if "Windows" == platform.system():
            env_set = 'set path={0}/bin;{0}/toolchain/bin;{1};'.format(self._path, os.getenv("Path"))
        else:
            env_set = 'export PATH={0}/bin:{0}/toolchain/bin:{1}'.format(self._path, os.getenv("PATH"))

        data = {
            "env": env_set,
            "builder": self.builder,
            "prjname": project.name,
            "target": target,
            "prjdir": project.prjdir,
            "workspace": kwargs.get('workspace'),
            "logfile": logfile,
        }

        buildcmd = '{env} && "{builder}" --launcher.suppressErrors -nosplash -application "org.eclipse.cdt.managedbuilder.core.headlessbuild" '\
        ' -cleanBuild "{prjname}/{target}" -import "{prjdir}" -data "{workspace}" >> {logfile} 2>&1'.format(**data)

        return buildcmd


    @staticmethod
    def get_latest():
        """Search and return a latest KDS instance in system

        Returns:
            <KDS.APP object>
        """
        path, version = get_latest()
        if path:
            return APP(path, version=version)
        else:
            return None


    @staticmethod
    def parse_build_result(exitcode, logfile):
        """Parse KDS build result

        Arguments:
            exitcode {int} -- the exit code of kinetis-design-studio process
            logfile {string} -- a file contains the output from kinetis-design-studio.

        Returns:
            <BuildResult object> -- the build result
        Notes:
        ---------------------
        kinetis-design-studio exit code
            noerror warnnings: 0
            erros: none-zero

        Parser the log file by search the keyword: "Total number of warnings: <>".
        """
        content = []
        result = BuildResult.Pass
        if 0 == exitcode:
            with open(logfile, "r") as f:
                content = f.read()
                if re.search(r"warning:", content):
                    result = BuildResult.Warnings
        else:
            result = BuildResult.Errors
        return result

    @staticmethod
    def verify(path):
        '''
        verify the path of compiler is avaliable
        '''
        return os.path.exists(path+"/eclipse/kinetis-design-studio")

    @property
    def is_ready(self):
        return os.path.exists(self.builder)

    @staticmethod
    def default_install_path(osname):
        default_path_table = {
            "Windows": "C:/Freescale",
            "Linux": "/opt/Freescale",
            "Darwin": "/Applications"
        }
        return default_path_table[osname]


def get_latest():
    """Get the latest kds version

    Returns:
        tuple -- (path, version)
    """
    osname = platform.system()
    kds_tool_list = []
    try:
        default_root_path = APP.default_install_path(osname)
        for per_folder in glob.glob(default_root_path + '/KDS**'):
            if "Darwin" == osname:
                kds_tool_list.append(os.path.join(default_root_path, per_folder, "Contents"))
            else:
                kds_tool_list.append(os.path.join(default_root_path, per_folder))
    except Exception:
        return None,None



    kds_versions = {}
    for per_kds in kds_tool_list:
        readmefile = per_kds + "/README.TXT"
        if not os.path.exists(readmefile):
            continue
        with open(readmefile, "r") as f:
            lines = f.readlines()
        for line in lines:
            match = re.search(r"\d\.\d\.\d", line)
            if match:
                kds_versions[match.group()] = per_kds
                break

    if kds_versions:
        latest_v = sorted(kds_versions.keys(), key=lambda v:LooseVersion(v))[-1]
        return kds_versions[latest_v], latest_v

    return None, None




import os
import re
import glob
import logging

from xml.etree import cElementTree as ET
from mcutk.apps.projectbase import ProjectBase




class Project(ProjectBase):
    """
    MDK uvsion project object

    This class could parser the settings in *.uvprojx.
    """

    PROJECT_EXTENSION = '*.uvprojx'

    OPTIONS_ROOT = 'Targets/Target/TargetOption/'


    BUILD_OPTIONS = {
        'TargetCommonOption/CreateHexFile': '1',
        'TargetCommonOption/BrowseInformation': '0',
        # 'TargetCommonOption/DebugInformation': '0',
        # 'TargetCommonOption/CreateLib': '0',
    }


    DEBUGGERS = {
    'cmsis-dap': {
            'DebugOption/TargetDlls/Driver': 'BIN\\CMSIS_AGDI.dll',
            'Utilities/Flash2': 'BIN\\CMSIS_AGDI.dll',
        },
    'j-link':{
            'DebugOption/TargetDlls/Driver': 'Segger\\JL2CM3.dll',
            'Utilities/Flash2': 'Segger\\JL2CM3.dll',
        }
    }




    DEBUGGING_OPTIONS = {
        'Utilities/Flash1/UpdateFlashBeforeDebugging' : '0',
        'DebugOption/Target/RestoreBreakpoints': '0',
        'DebugOption/Target/RestoreWatchpoints': '0',
        'DebugOption/Target/RestoreMemoryDisplay': '0',
        'DebugOption/Target/RestoreFunctions': '0',
        'DebugOption/Target/RestoreToolbox': '0',
        'DebugOption/Target/RestoreTracepoints': '0',
        'DebugOption/Target/RestoreSysVw': '0',
    }



    def __init__(self, *args, **kwargs):
        super(Project, self).__init__(*args, **kwargs)
        try:
            if not self.prjpath.endswith(".uvprojx"):
                logging.warning("%s: is not a project file", self.prjpath)
                dirname = os.path.dirname(args[0])
                prjpath = glob.glob(dirname+"/*.uvprojx")[0]
                logging.warning("we have automaticly loaded the file to replace: %s", prjpath)
                self.prjpath = prjpath
        except IndexError:
            raise ValueError("Not found the project file .uvprojx under project parent directory!")
        # .uvprojx
        self.prjpath = self.prjpath.replace('\\', '/')
        # .uvoptx
        self.prjuvoptx = self.prjpath.replace('.uvprojx', '.uvoptx')
        self._prjxmltree = ET.parse(self.prjpath)
        self._conf = self._configurations()
        self._targets = self._conf.keys()
        # simplify targets: remove project name
        self._simple_targets = [t.replace(self.name, '').strip() for t in self._targets]


    def _configurations(self):
        targets = dict()
        all_targets = self._prjxmltree.findall("Targets/Target")
        for target_node in all_targets:
            target_name = target_node.find("TargetName").text.strip()
            output_dir = target_node.find("TargetOption/TargetCommonOption/OutputDirectory").text.strip()
            output_name = target_node.find("TargetOption/TargetCommonOption/OutputName").text.strip()
            targets[target_name] = output_dir + output_name
        return targets



    def to_dict(self):
        # use simple targets
        return {
            'toolchain': self.idename,
            'targets': self._simple_targets,
            'project': self.prjpath,
            'name': self.name
        }




    @property
    def targetsinfo(self):
        """Returns a dict about the targets data.

        Example:
        {
            "Debug":   "debug_output_dir/output_name",
            "Release": "release_output_dir/output_name",
        }
        """
        return self._conf


    @property
    def name(self):
        """Return the application name

        Returns:
            string --- app name
        """
        return os.path.basename(self.prjpath).split(".")[0]


    def _set_value(self, nodepath, value):
        """Update value for specific nodes."""
        for node in self._prjxmltree.findall(nodepath):
            node.text = value


    def get_flash_driver(self, target):
        nodepath = "./Targets/Target[TargetName='{}']/TargetOption/TargetCommonOption/FlashDriverDll".format(target)
        return self._prjxmltree.find(nodepath).text



    def update_inifile(self, target):
        """Append 'EXIT' command to initializationfile to make sure the debugging exit when flash
        finished. If InitializationFile is not configured, this will set a default file
        'debugging.ini'.
        """
        nodepath = "./Targets/Target[TargetName='{}']/TargetOption/DebugOption/TargetDlls/InitializationFile".format(target)
        root = os.path.dirname(self.prjpath)
        try:
            _update_inifile(self._prjxmltree, root, nodepath)
        except ValueError:
            pass



    def set_build_options(self):
        """Set options to control keil build behaviour.
        1. Disable browse information will speed up the build.
        2. Create hex file.
        """
        for path, value in self.BUILD_OPTIONS.items():
            self._set_value(self.OPTIONS_ROOT + path, value)
        self.save()


    def set_debugging_options(self, target, sn='', debugger_type='cmsis-dap', debugfile=None, action='program'):
        """Set debugging options to enable the automated flash programming by keil.

        Arguments:
            debugger_type -- {str} -- debugger type, avalibe choices are
                'cmsis-dap', 'j-link', default is 'cmsis-dap'.
            action: 'program' or 'erase'
        """
        if not os.path.exists(self.prjuvoptx):
            logging.warning('.uvoptx file is not exists, create new from template!!!!')
            Uvoptx.create_new_from_template(self.prjuvoptx, target)

        optx = Uvoptx(self.prjuvoptx)
        optx.set_debugger(target, debugger_type)
        optx.set_cmsis_serial_number(target, sn, action)
        inifile = optx.update_inifile(target)

        debug_mode = False
        is_to_flash = True

        # exclude xip ini file, which is required as external flash, not for RAM
        if 'flexspi_nor' in target or inifile == None or "xip" in inifile:
            debug_mode = False
            is_to_flash = True

        elif inifile != None:
            debug_mode = True
            is_to_flash = False


        optx.save()

        # check target
        # target = self.map_target(target)
        # update .uvprjx
        self.update_inifile(target)

        for path, value in Project.DEBUGGING_OPTIONS.items():
            if 'UpdateFlashBeforeDebugging' in path:
                value = '1' if is_to_flash else '0'
            self._set_value(self.OPTIONS_ROOT + path, value)

        # Force to update debugger
        for path, value in Project.DEBUGGERS[debugger_type].items():
            self._set_value(self.OPTIONS_ROOT + path, value)

        if debugfile:
            self._set_value(self.OPTIONS_ROOT + 'TargetCommonOption/OutputDirectory', os.path.dirname(debugfile) + '\\')
            self._set_value(self.OPTIONS_ROOT + 'TargetCommonOption/OutputName', os.path.basename(debugfile))

        self.save()
        return debug_mode





    def save(self):
        """Save changes"""
        self._prjxmltree.write(self.prjpath, xml_declaration=True, method='xml', encoding='UTF-8')







class Uvoptx(object):

    DEBUGGERS = {
        'cmsis-dap': 'BIN\\CMSIS_AGDI.dll',
        'j-link':'Segger\\JL2CM3.dll',
    }

    TEMPLATE = os.path.join(os.path.abspath(os.path.dirname(__file__)), \
        'templates/mdk.uvoptx')

    @classmethod
    def create_new_from_template(cls, uvoptx_path, target):
        tree = ET.parse(cls.TEMPLATE)
        tree.find('./Target/TargetName').text = target
        tree.write(uvoptx_path, xml_declaration=True, method='xml', encoding='UTF-8')


    def __init__(self, path):
        self.path = path
        self._xmltree = ET.parse(path)


    def set_debugger(self, target, debugger='cmsis-dap'):
        nodepath = "./Target[TargetName='{}']/TargetOption/DebugOpt/pMon".format(target)
        self._xmltree.find(nodepath).text = self.DEBUGGERS[debugger]



    def update_inifile(self, target):
        nodepath = "./Target[TargetName='{}']/TargetOption/DebugOpt/tIfile".format(target)
        root = os.path.dirname(self.path)
        return _update_inifile(self._xmltree, root, nodepath)


    def set_cmsis_serial_number(self, target, sn, action='program'):
        """Set CMSIS-DAPLINK serial number into .uvoptx.

        action: program/erase.
        """
        parent_path = "./Target[TargetName='{}']/TargetOption/TargetDriverDllRegistry".format(target)
        # get parent node
        parent_node = self._xmltree.find(parent_path)

        if parent_node is None:
            parent_node = ET.fromstring('<TargetDriverDllRegistry></TargetDriverDllRegistry>')
            self._xmltree.find("./Target[TargetName='{}']/TargetOption/".format(target)).append(parent_node)

        # flash_driver = flash_driver.replace('UL2CM3(', '').replace('.FLM))', '.FLM)')
        # get specfic node
        node = parent_node.find("./SetRegEntry[Key='CMSIS_AGDI']")
        if node is None:
            raise ValueError('Error: project debugger is not cmsis-dap!')
            # not exists, this will use default blocks
            # node = ET.fromstring(default_blocks)
            # parent_node.append(node)

        # update sn value
        driver = node.find('Name').text
        sn = sn[:32]
        snvalue = '-U{}'.format(sn)

        snp = re.compile(r'-U\w*')
        if snp.search(driver) is not None:
            driver = snp.sub(snvalue, driver)
        else:
            driver = snvalue + ' ' + driver # put the sn number at the beginning to avoid compatible issues


        # -FO1 means: erase flash
        # -FO15 means: program, verify, reset and run
        # if not do that the program will not run after flash download
        # -FO15 means: program, verify, reset and run
        # if not do that the program will not run after flash download
        # -F011 means: program, reset and run (without verify)
        # The Aruba Flash less requires this F011, and we will set F011 as default options since it's more stable
        if action == 'program':
            ac_value = ' -FO11 '
        elif action == 'erase':
            ac_value = ' -FO1 '
        flash_settings = re.compile(' -FO\d+ ')
        if flash_settings.search(driver) is not None:
            driver = flash_settings.sub(ac_value, driver)
        else:
            driver += ' -FO11 '
            logging.warning('flash setting not detected.')

        node.find('Name').text = driver
        logging.debug(node.find('Name').text)




    def save(self):
        self._xmltree.write(self.path, xml_declaration=True, method='xml', encoding='UTF-8')



def _update_inifile(tree, project_root, nodepath):
    # default_ini = os.path.join(os.path.abspath(os.path.dirname(__file__)), \
    #     'debugging.ini').replace('\\', '/')

    node = tree.find(nodepath)
    if node is None:
        logging.debug('Warning: The path "%s" is not exists!'%nodepath)
        return None

    # not conigured file, means it run in ram or sdram.
    if not node.text:
        return None
        #node.text = default_ini


    # configured but it maybe a relative path, further check
    elif not os.path.exists(node.text):
        inifile = os.path.join(project_root, node.text)
        if not os.path.exists(inifile):
            raise IOError('InitializationFile(%s) is not exists!'%inifile)

        with open(inifile, 'r') as file:
            content = file.read()

        # Append "EXIT" command to inifile
        if 'EXIT' not in content:
            with open(inifile, 'a+') as file:
                file.write('EXIT')

    logging.debug('InitializationFile: %s', node.text)
    return node.text



import serial
import pexpect
import sys
import time

sys.path.append("./../")

from mcutk.pserial import Serial
from mcutk.board import getboard
from mcutk.debugger.jlink import JLINK


jlinkd = JLINK.get_latest()
print jlinkd


board = getboard(
    "frdmkl43z",
    devicename="MKL43Z256xxx4",
    usbid="621000000",
    debugger_type="jlink"
    )

board.debugger = jlinkd
board.debugger.reset()

time.sleep(1)
ser = Serial('COM20', 9600, timeout=0.1)
ser.start_reader()
time.sleep(1)
ser.write("A")
time.sleep(1)
print ser.data

spawn = ser.Spawn()

interacts = [
    {
        "input": "A",
        "expect": r"Waiting for power.mode select\.\."
    },
    {
        "input": "B",
        "expect": r"Select the wake up source.*Waiting for key press\.\."
    },
    {
        "input": "T",
        "expect": r"Select the wake up timeout in seconds.*Waiting for input timeout value",
    },
    {
        "input": "1",
        "expect": r"Waiting for power.mode select.",
    },
]

# spawn.logfile_read = sys.stdout

for item in interacts:
    spawn.test_input(**item)
    time.sleep(0.2)

spawn.flush_log()
print ser.data

spawn.expect([pexpect.EOF, pexpect.TIMEOUT])
spawn.close()






from __future__ import print_function
import click
import glob
import os
from os.path import join

from mcutk.debugger import getdebugger
from mcutk.managers.conf_mgr import ConfMgr


DEBUGGERS = ['jlink', 'pyocd']


def find_debuggers():
    """Find installed debuggers from system."""

    all_debuggers = list()
    for name in DEBUGGERS:
        debugger = getdebugger(name).get_latest()
        if debugger and debugger.is_ready:
            all_debuggers.append(debugger)
        else:
            click.echo('Error, debugger %s is unavailiable to use!'%name)
    return all_debuggers


def list_devices(debuggers):
    """List connected devices."""

    devices = list()
    for deb in debuggers:
        for item in deb.list_connected_devices():
            devices.append(item)
    return devices


def select_device(debuggers, devices):
    """Promt user to select device."""

    selected_device = None
    selected_debugger = None

    if len(devices) == 0:
        print('no devices found.')
        return None, None

    elif len(devices) == 1:
        selected_device = devices[0]
    else:
        for index, item in enumerate(devices):
            print('{:2} - {:10} - {} - {}'.format(index, item['type'], item.get('name'), item['usbid']))

        while True:
            try:
                index = raw_input('Please input the index to select device > ')
                index = int(index)
                selected_device = devices[index]
                break
            except Exception:
                pass

    if selected_device['debugger'] == 'jlink':
        selected_device['devicename'] = raw_input('Please input device name > ')

    debugger_type = selected_device['debugger']
    for deb in debuggers:
        if deb.name == debugger_type:
            selected_debugger = deb
            break
    else:
        print("Error: unknown debugger type: %s"%debugger_type)

    return selected_debugger, selected_device



from __future__ import print_function
import os
import glob
import logging
import platform


from .app import APP
from mcutk.util import run_command
from mcutk.apps.decorators import appinstall





@appinstall
def install(installer, **kwargs):
    """Install Armgcc to system, and installaton verification.

    Arguments:
        installer {string} -- path to installer file

    Returns:
        tuple -- (path, version)
    """
    if _install(installer) != 0:
        return None, None

    print("Verify installation...")
    _armgcc  = APP.get_latest()
    if _armgcc:
        return _armgcc.path, _armgcc.version

    return None, None





def _install(installer):
    """Install armgcc to system

    Arguments:
        installer {string} -- path to installer file

    Returns:
        tuple -- (path, version)
    """

    default_root = {
            "Windows": "C:/Program Files (x86)/GNU Tools ARM Embedded",
            "Linux": "/usr/local",
            "Darwin": "/usr/local",
        }
    osname = platform.system()
    install_path = default_root.get(osname)

    if osname == "Windows":
        return os.system("start /wait {0} /S /NCRC".format(installer))

    elif osname == "Linux" or osname == "Darwin":
        return os.system("tar -jxvf {1} -C {2}".format( installer, install_path))



from mcutk.exceptions import ToolchainError
from mcutk.apps import appfactory



def build_project(tool_name, tool_path, project_path, tool_version=None,
    target=None, logfile=None, workspace=None, timeout=300):
    """A simple shortcut method to build C/C++ projects for embedded toolchains.

    Arguments:
        tool_name {string} -- toolchain name, like: iar, mdk, mcux, armgcc...
        tool_path {string} -- toolchain installation root directory
        project_path {string} -- project root directory or project path
        workspace {string} -- workspace directory for Eclipse based toolchains
        timeout {int} -- timeout for build
        tool_version {string} -- set toolchain version
        target {string} -- project configuration, debug/release
        logfile {string} -- log file path (default: {None})

    Raises:
        ToolchainError -- raise on unsupported toolchain
        IOError -- raise on toolchain is not ready

    Returns:
        mcutk.apps.BuildResult -- build result
    """

    tool = appfactory(tool_name)
    if not tool:
        raise ToolchainError('unsupported tool: %s'%tool_name)

    toolchain = tool(tool_path, version=tool_version)
    if not toolchain.is_ready:
        raise IOError('tool is not ready to use, please check the tool path!')

    project = tool.Project.frompath(project_path)

    if target:
        target = project.map_target(target)
    else:
        target = project.targets[0]

    ret = toolchain.build_project(project, target, logfile, workspace=workspace, timeout=timeout)

    return ret






from __future__ import print_function
import os
import re
import glob
import time
import platform
import zipfile
import subprocess
import logging
import multiprocessing
import os.path as Path
from distutils.version import LooseVersion

from mcutk.apps.decorators import build
from mcutk.apps.idebase import IDEBase, BuildResult
from mcutk.util import copydir, run_command

class APP(IDEBase):
    """
    IAR Workbench interface implementation.
    This class inherit from IDEBase, and it has the ability to build project,
    and parse the build result.

    Create an instance:
        (1) search in windows registry:
            >>> iar = APP.get_latest()
            >>> print iar.version

        (2) from api
            >>> iar = APP("C:/Program Files(x86)/IAR systems/IAR Workbench-8.11/", version="8.11")
            >>> print iar.version

    """

    OSLIST = ["Windows"]


    EXITCODE = {
        0:	"PASS",
        1:	"Warnings",
        2:	"Errors",
        3:	"Fatal Errors",
    }



    @staticmethod
    def get_latest():
        """Search and return a latest IAR instance in system

        Returns:
            <iar.APP object>
        """
        if os.name == 'nt':
            path, version = get_latest()
            if path:
                return APP(path, version=str(version))
        return None


    @staticmethod
    def parse_build_result(exitcode, logfile):
        """Parse iar build result

        Arguments:
            exitcode {int} -- the exit code of iarbuild process
            logfile {string} -- a file contains the output from iarbuild.

        Returns:
            <BuildResult object> -- the build result


        Notes:
        ---------------------
        iarbuild.exe exit code
            noerror warnnings: 0
            erros: none-zero

        Parser the log file by search the keyword: "Total number of warnings: <>".
        """
        result = "error"

        if exitcode == 0:
            result = "pass"
            if logfile:
                with open(logfile, "r") as fobj:
                    content = fobj.read()
                s = re.compile(r'Total number of warnings:\s*[1-9]', re.I).search(content)
                result = "warning" if s else "pass"

        return BuildResult.map(result, )





    @staticmethod
    def verify(path):
        '''
        verify the path of compiler is avaliable
        '''
        return Path.exists(path+"/common/bin/IarIdePm.exe")





    def __init__(self, *args, **kwargs):
        super(APP, self).__init__("iar", *args, **kwargs)
        self.builder = Path.join(self._path, "common", "bin", "IarBuild.exe")




    def __str__(self):
        return "IAR-" + self.version



    @property
    def is_ready(self):
        return Path.exists(self.builder)





    def enable_license(self, value):
        """
        Enable iar network license
        """
        if os.system("\"{0}/common/bin/LightLicenseManager.exe\" -s {1} -p ARM.EW"
                     .format(self.path, value)) == 0:
            self.check_license()
            return True
        else:
            return False



    def check_license(self):
        """
        Open LicenseManager.exe, wait for it communicate with license server.
        """
        proc = subprocess.Popen(["{0}/common/bin/LicenseManager.exe".format(self.path)])
        time.sleep(30)
        proc.kill()




    @build
    def build_project(self, project, target, logfile, **kwargs):
        """Return a string about the build command line.

        Arguments:
            project {iar.Project} -- iar project object
            target {string} -- target name
            logfile {string} -- log file path

        Raises:
            IOError -- [description]

        Returns:
            string -- build commandline



        IAR command line guide is quoted from IAR Workbench reference:

            To build the project from the command line, use the IAR Command Line Build Utility
            (iarbuild.exe) located in the common\bin directory. As input you use the project
            file, and the invocation syntax is:

                iarbuild project.ewp [-clean|-build|-make] <configuration>
                [-log errors|warnings|info|all]

            -----------------------------------
            Parameter           Description
             -----------------------------------
            project.ewp     Your IAR Embedded Workbench project file.
            -clean          Removes any intermediate and output files.
            -build          Rebuilds and relinks all files in the current build configuration.
            -make           Brings the current build configuration up to date by compiling,
                            assembling, and linking only the files that have changed since the last
                            build.
            configuration   The name of the configuration you want to build, which can either be
                            one of the predefined configurations Debug or Release, or a name that
                            you define yourself. For more information about build configurations, see
                            Projects and build configurations, page 32.
            -log errors     Displays build error messages.
            -log warnings   Displays build warning and error messages.
            -log info       Displays build warning and error messages, and messages issued by the
                            #pragma message preprocessor directive
            -log all        Displays all messages generated from the build, for example compiler
                            sign-on information and the full command line.
        """

        try:
            if not project.prjpath.endswith(".ewp"):
                projpath = glob.glob(Path.dirname(project.prjpath)+"/*.ewp")[0]
            else:
                projpath = project.prjpath
        except Exception:
            raise IOError("Cannot find *.ewp file, iarbuild cannot build!")

        cpu_count = multiprocessing.cpu_count()
        buildcmd = '"{0}" "{1}" -build "{2}" -log info -parallel {3}'.format(
            self.builder,
            projpath,
            target,
            cpu_count)

        if logfile:
            buildcmd += ' >> "%s" 2>&1'%(logfile)

        return buildcmd




    def install_pack(self, packpath, tmpdir):
        zip_ref = zipfile.ZipFile(packpath, 'r')
        zip_ref.extractall(tmpdir)
        zip_ref.close()
        logging.info("Scucessfully unziped!")

        items = [i for i in os.listdir(tmpdir) if i not in ("binary", "logs")]
        iararm = self.path+"/arm"

        if len(items) == 1:
            foldername = items[0]
            folderpath = tmpdir+"/"+foldername
            print("unzip dir: %s"%folderpath)
            sopurce_path = folderpath
            dest_path = None

            if "arm" in foldername:
                if foldername != "arm":
                    sopurce_path = Path.join(tmpdir, "arm").replace("\\", "/")
                    os.rename(folderpath, sopurce_path)

                dest_path = iararm

            elif "config" in foldername:
                if foldername != "config":
                    sopurce_path = Path.join(tmpdir, "config").replace("\\", "/").replace("//", "/")
                    os.rename(folderpath, sopurce_path)

                dest_path = iararm+"/config"
            else:
                raise NotImplementedError("Unknown structure tree!")

            copydir(sopurce_path, dest_path)
            logging.info("Successfully installed!")
        else:
            raise NotImplementedError("Not supported format")


    def programming(self, board, prjdir, target, file, **kwargs):
        """This will use IAR Cspybat.exe to download debugfile.

        In order to simple the cspybat, this function just replace the iar root,
        and update the usb serial number.

        Arguments:
            prjdir {str} -- project root directory
            target {str} -- target name
            board {mcutk.board.Board} Board object
            debugfile {str} -- path to debug file(image)
            timeout {int}  -- time out

        Raises:
            ValueError -- Raise when not found *.xcl files

        Returns:
            int -- cspybat exit code
        """
        timeout = kwargs.get('timeout', 120)
        return _cspy_programming(self.path, prjdir, board.usbid, target, file, timeout)



    def transform_elf(self, type, in_file, out_file):
        """IAR ELF Tool.
        This method actually will call <iar_root>/arm/bin/ielftool.exe.
        """
        return _elftool(self.path, type, in_file, out_file)




def _elftool(iar_root, type, in_file, out_file):
    """IAR ELF Tool.

    Arguments:
        iar_root {str} -- [description]
        type {str} -- [description]
        in_file {str} -- [description]
        out_file {str} -- [description]

    Raises:
        IOError -- raise when ielftool.exe does not exists.
        ValueError -- raise when type is unsupported.
    """

    elftool = Path.join(iar_root, 'arm/bin/ielftool.exe')
    if not Path.exists(elftool):
        raise IOError('arm/bin/ielftool.exe: does not exists!')

    supported_types = ['bin', 'srec', 'ihex']

    if type not in supported_types:
        raise ValueError('unsupported format type!')

    elftool_command = [
        elftool,
        in_file,
        out_file,
        "--"+type
    ]

    return subprocess.call(elftool_command, shell=True) == 0




def _cspy_programming(iar_root, prjdir, usbid, target, debugfile, timeout):
    """This will use IAR Cspybat.exe to download debugfile.

    In order to simple the cspybat, this function just replace the iar root,
    and update the usb serial number.

    Arguments:
        iar_root {str} -- the root directory of IAR
        prjdir {str} -- project root directory
        usbid {str} -- debug probe serial number
        target {str} -- app target name
        debugfile {str} -- path to debug file(image)

    Raises:
        ValueError -- Raise when not found *.xcl files

    Returns:
        int -- cspybat exit code
    """
    general_xcl = _update_xcl_file(iar_root, prjdir, target, '/*.%s.general.xcl'%target)
    driver_xcl = _update_xcl_file(iar_root, prjdir, target, '/*.%s.driver.xcl'%target)

    # a workround for https://jira.sw.nxp.com/browse/KPSDK-21428
    # TODO: wait for iar fix this issue
    #usbid = usbid[-8:]

    iar_root = iar_root.replace('\\', '/')
    cspybat_command = r'"{0}\\common\\bin\\cspybat.exe" -f "{1}" "--debug_file={2}"'\
    ' --leave_target_running --timeout 500 --backend -f "{3}" "--drv_communication=USB:#{4}"'.format(
        iar_root,
        general_xcl,
        debugfile,
        driver_xcl,
        usbid
    )
    ret = run_command(cspybat_command, stdout=True, timeout=timeout)
    logging.info('iar cspybat.exe exit code: %s', ret[0])

    return ret


def _update_xcl_file(iar_root, prjdir, target, glob_pattern):
    """Replace iar_root and project path in xcl file."""
    str_root_pattern = r"(C:.Program.Files.*IAR Systems\\Embedded Workbench[\w\W]*\\arm\\)|(C:.PROGRA.*IARSYS~1.EMBEDD~.*(\/|\\)arm\/)|(C:.PROGRA.*IAR.*(\/|\\)arm\/)"
    str_usbid_pattern = "--drv_communication=.*\n"
    str_path_pattern = r'[a-zA-Z]\:.*(\\|\/)'
    root_pattern = re.compile(str_root_pattern, re.I)
    usbid_pattern = re.compile(str_usbid_pattern, re.M)
    path_pattern = re.compile(str_path_pattern, re.M)

    settings_dir= Path.join(prjdir, 'settings')
    xclfiles = glob.glob(settings_dir + glob_pattern)
    if not xclfiles:
        raise ValueError("not found *.general.xcl")

    # find the "*.xcl"
    for xcl in xclfiles:
        if target in xcl:
            xclfile = xcl
            break
    else:
        xclfile = xclfiles[0]

    iar_root += r'\\arm/'
    # update xcl content
    with open(xclfile, "r+") as f:
        lines = f.readlines()
        for idx, line in enumerate(lines):
            # inspect the iar_root directory
            if root_pattern.search(line) is not None:
                line = root_pattern.sub(iar_root, line)

            # update project path
            elif path_pattern.search(line) is not None:
                # python3 need to escape the path
                prjdir = prjdir.replace('\\', '/')
                line = path_pattern.sub(prjdir + "/", line)

            # remove the usb id in xcl file
            if usbid_pattern.search(line) is not None:
                line = ''
            # disable verify step
            elif "--drv_verify_download" in line:
                line = ''

            lines[idx] = line

        content = ''.join(lines)
        f.seek(0)
        f.truncate()
        f.write(content)

    return xclfile



def get_all_instance():
    """Get all installed iar instance from local PC.

    Returns:
        list of tuple -- [(path1, version1), ...]
    """
    try:
        import _winreg as winreg
    except ImportError:
        import winreg

    # may more than one iar info on one PC
    index = 0
    version_pool = []
    # windows 32bit
    if platform.architecture()[0] == "32bit":
        root_key = r"SOFTWARE\IAR Systems\Embedded Workbench\5.0\EWARM"
    else:
        root_key = r"SOFTWARE\Wow6432Node\IAR Systems\Embedded Workbench\5.0\EWARM"

    try:
        key_object = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, root_key, 0, winreg.KEY_READ)
    except WindowsError as e:
        print("Error: Not found IAR instance in your system!")
        return version_pool



    # walk all installed iar
    while True:
        iar_path = "Unknown"
        iar_version = "Unknown"
        try:
            #enum sub_key name
            sub_key_name = winreg.EnumKey(key_object, index)
            #get sub key path
            sub_key_path = root_key + "\\" + sub_key_name
            #open this path
            sub_key_object = winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE,
                sub_key_path,
                0,
                winreg.KEY_READ)
            #QueryEx return a tuple (value, type)
            iar_path = winreg.QueryValueEx(sub_key_object, "InstallPath")[0].replace('\\', '/')
            iar_version = sub_key_name

        except WindowsError as e:
            if "Access is denied" in str(e):
                raise e
            break

        if iar_version != "Unknown":
            version_pool.append((iar_path, iar_version))
        index += 1

    return version_pool



def get_latest():
    """Get the latest iar version

    Returns:
        tuple -- (path, version)
    """
    version_pool = get_all_instance()
    if version_pool:
        return _get_max_version(version_pool)
    else:
        return None, None





def _get_max_version(version_pool):
    versions = [(ver[0], LooseVersion(ver[1])) for ver in version_pool]
    versions.sort(key=lambda x:x[1])
    return versions[-1]






import os
import glob
import logging
import tempfile
from distutils.version import LooseVersion
from xml.etree import cElementTree as ET
from difflib import SequenceMatcher
from mcutk.exceptions import ProjectNotFound, ProjectParserError
from mcutk.apps.projectbase import ProjectBase
from mcutk.apps import eclipse


class Project(eclipse.Project):
    """LPCxpresso project parser."""


    @classmethod
    def frompath(cls, path):
        """Return a project instance from a given file path or directory.

        If path is a directory, it will search the project file and return an instance.
        Else this will raise mcutk.apps.exceptions.ProjectNotFound.
        """
        if os.path.isfile(path):
            return cls(path)

        instance = None
        for file in glob.glob(path + "/.project"):
            filepath = os.path.abspath(file)
            if "lpcx" in filepath:
                drivers = filepath.replace("\\", "/").split("/")
                if "lpcx" in drivers:
                    instance = cls(file)
                    break
        return instance



    def __init__(self, prjpath, **kwargs):
        """LPCX project constructor.

        Arguments:
            prjpath {str} -- path to .cproject/.project

        """
        super(Project, self).__init__(prjpath, **kwargs)
        self.parse(self.prjpath)












from __future__ import print_function
import os
import glob
import shutil
import platform
import logging

from .app import APP
from mcutk.apps.decorators import appinstall
from mcutk.util import run_command


@appinstall
def install(installer, **kwargs):
    """Install mcuxpressoide to system, and verify the installation

    Arguments:
        installer {string} -- path to installer file

    Returns:
        tuple -- (path, version)
    """
    rc = _install(installer)
    print("  Finished run setup.exe, exit code: %s"%rc)
    if rc != 0:
        return None, None

    print("Verify installation...")
    _mcux  = APP.get_latest()

    if _mcux:
        return _mcux.path, _mcux.version

    return None, None









def _install(installer):
    """Silent installation

    Description:
        - Windows:
            MCUXpressoIDE is packed by inno tool, inno silent installation guide:
            >>> setup.exe /SILENT /DIR=<path> /NORESTART /SUPPRESSMSGBOXES /SP-

        - Linux:
            1. Unzip installer:
                >>> **.deb --noexec --target <Uncompress directory>
            2. Assign executable permission.
            3. Use dpkg to install .deb:
                >>> dpkg -i --force-all --force-depends <path to **.deb>

        - Mac:
            >>> installer -store -pkg <path to installer> -target /

    """
    osname = platform.system()
    if osname == "Windows":
        install_cmd = "start /wait {0} /SILENT /NORESTART /SUPPRESSMSGBOXES /SP-".format(installer)
        ret = run_command(install_cmd, timeout=10)[0]

        # Tiemout function only killed the cmd.exe process.
        # If mcux installer is blocked by window prompt, below
        # command will kill the real installer process to make sure it exit.
        os.system('''TASKKILL /F /T /FI" IMAGENAME eq MCUXpressoIDE_*"''')
        return ret


    elif osname == "Linux":
        setupname = os.path.basename(installer)
        # uncompress the .deb.run or .deb.bin to uncompress_dir
        uncompress_dir = os.path.join(os.path.dirname(installer), setupname.replace(".", "_"))

        # check & cretae uncompress directory
        if os.path.exists(uncompress_dir):
            shutil.rmtree(uncompress_dir, ignore_errors=True)
        os.makedirs(uncompress_dir)

        # make sure the installer has execution permission
        os.chmod(installer, 0o777)
        # extract the files from installer
        ret = os.system("{} --noexec --target {}".format(installer, uncompress_dir))
        print("Uncompressing done, exit code: %s\n"%ret)

        # Following code is commented, we will not check the return code from uncompressing.
        # But to check the *.deb file.
        # if ret != 0:
        #     print("Uncompressing error: %s"%setupfile)
        #     return return_code

        files = glob.glob(uncompress_dir + "/*.deb")
        deb_filpath = files[0] if files else None
        if deb_filpath:
            print("Error: *.deb is not exists, probably the uncompress failed!")
            return 1

        os.chmod(deb_filpath, 0o777)
        ret = os.system("dpkg -i --force-all --force-depends {}".format(deb_filpath))

        if ret != 0:
            print("dpkg install error %s"%deb_filpath)
            return ret

        # delete uncompress directory
        shutil.rmtree(uncompress_dir, ignore_errors=True)

        os.system("apt-get -fy install --force-yes")
        return 0


    elif osname == "Darwin":
        os.chmod(installer, 0o777)
        cmd = "installer -store -pkg {} -target /".format(installer)
        return os.system(cmd)






#!/usr/bin/env python

import os
import sys
import copy
import logging
import os.path as Path

import subprocess
from mcutk.exceptions import ReadElfError

from mcutk.appbase import APPBase
from mcutk.apps import appfactory

from elftools.elf.elffile import ELFFile
from elftools.common.exceptions import ELFError
from elftools.elf.sections import SymbolTableSection
from elftools.elf.descriptions import  describe_p_flags




def transform_elf_basic(type, in_file, out_file, executor=None):
    """Transform ELF to specific type by arm-none-eabi-objcopy.

    Supported types: bin, ihex, srec.

    Arguments:
        type {str} -- which type you want to convert.
        in_file {str} -- path to elf file.
        out_file {str} -- output file
        executor {str} -- path to arm-none-eabi-objcopy, if it is None,
                    program will use the default executor(mcutk/bin/)


    Raises:
        ReadElfError -- Unknown elf format will raise such error
        Exception -- Convert failed will raise exception

    Returns:
        bool
    """
    supported = {
        'bin': "binary",
        'ihex': 'ihex',
        'srec': 'srec'
    }

    if type not in supported:
        raise ValueError("unknown type, valid choices are: %s"%(str(supported)))

    if executor is None:
        executor = Path.join(Path.dirname(__file__), 'bin/arm-none-eabi-objcopy.exe')

    try:
        with open(in_file, 'rb') as fileobj:
            elffile = ELFFile(fileobj)
            for header_name in elffile.header:
                print('{:<10s} -- {:<10s}'.format(header_name, str(elffile.header[header_name])))
    except ELFError:
        raise ReadElfError('read elf error!')

    if not os.path.isfile(executor):
        raise IOError('arm-none-eabi-objcopy does not exists!')

    type = supported.get(type, type)
    cmds = '{0} -O {1} {2} {3}'.format(executor, type, in_file, out_file)

    return subprocess.call(cmds, shell=True) == 0




def transform_elf(ide, type, in_file, out_file):
    """Transform ELF to specific type with toolchain method (app.transform_elf).

    Supported types: bin, ihex, srec.

    Arguments:
        ide {str} -- ide name or ide app instance.
        type {str} -- which type you want to convert.
        in_file {str} -- path to elf file.
        out_file {str} -- output file

    Raises:
        ReadElfError -- Unknown elf format will raise such error
        Exception -- Convert failed will raise exception

    Returns:
        bool -- success or not.
    """
    if isinstance(ide, APPBase):
        app_instance = ide
    else:
        app_instance = appfactory(ide).get_latest()

    if not (app_instance and app_instance.is_ready):
        raise ValueError('Fatal error, not found toolchain "%s" in system!'%ide)

    return app_instance.transform_elf(type, in_file, out_file)







# class ELFTool:
#     def __init__(self, elffile):
#         self.elf = elffile

#     def get_elf_symbol_by_name(self, name):
#         target_symbol = list()
#         elffile = None
#         with open(self.elf, 'rb') as f:
#             elffile = ELFFile(f)
#             for section in elffile.iter_sections():
#                 if not isinstance(section, SymbolTableSection):
#                     continue
#                 for i, sym in enumerate(section.iter_symbols()):
#                     if sym.name == name:
#                         target_symbol.append(sym)

#         for per_sym in target_symbol:
#             per_sym.entry['st_value'] = hex(per_sym.entry['st_value'])
#         return target_symbol

#     def get_elf_program_headers(self):
#         program_header = {}
#         elffile = None
#         with open(self.elf, 'rb') as f:
#             elffile = ELFFile(f)
#             program_header['entry_point'] = hex(elffile.header["e_entry"])
#             for i, segment in enumerate(elffile.iter_segments()):
#                 program_header['segment{}'.format(i)] = {
#                     'p_paddr': hex(segment['p_paddr']),
#                     'p_vaddr': hex(segment['p_vaddr']),
#                     'p_memsz': hex(segment['p_memsz']),
#                     'p_filesz': hex(segment['p_filesz']),
#                     'p_align': segment['p_align'],
#                     'p_flags': describe_p_flags(segment['p_flags'])
#                 }
#         return program_header



from __future__ import print_function
import os
import glob
import logging

from .app import APP, get_latest
from mcutk.apps.decorators import appinstall


@appinstall
def install(installerfile, tmpdir, license):
    """install iar

    Arguments:
        installerfile {string} -- path to installer file
        tmpdir {string} -- path to tmp directory

    Returns:
        tuple -- (path, version)
    """
    rc = _install(installerfile, tmpdir)
    logging.info("finished to install, process exit code: %s", rc)
    if rc != 0:
        logging.error("Error: Installation is abnormal exit!")
        return None

    logging.info("Verify installation...")
    path, version = get_latest()


    if APP.verify(path):
        logging.info("Successfully install")
    else:
        logging.error("Verifcation is failure, probably the installation issue!")
        return None


    logging.info("Enable license...")
    iar = APP(path, version=version)
    if iar.enable_license(license):
        logging.info("Successfully enabled network license")
        return path, version
    else:
        logging.warning("Failed to enable iar license")
        return None




def _install(installerfile, tmpdir):
    '''
    Install iar on windows.

    IAR installer is packed with rar self-extract format. First of all,
    we need to use "UnRAR.exe" to extract the real installer "setup.exe".
    The setup.exe is assembled by installshield.

    Notice:
        1. setup.exe: you'd better not rename this file, if you do that, please
        don't include the number in the name! It will failed when the name
        contain number.

        2. *.iss
        The "*.iss" is a configuration for silent installation progress.
        To generate this file just run below command to record it.
            >> setup.exe /r
        Then the setup.iss will be generated and will be saved in C:/Windows.

    Arguments:
        installerfile: path of installer
        tmpdir: tmp directory

    Returns:
        int --- exit code
    '''
    curdir = os.path.dirname(os.path.abspath(__file__)).replace("\\", "/")
    unrar_tool = os.path.abspath(curdir + "../../bin/UnRAR.exe")

    # Extract setup.exe from installerfile
    cmd = "{0} -y {1} ew {2}".format(unrar_tool, installerfile, tmpdir)
    logging.info(cmd)
    ret = os.system(cmd)
    if ret != 0:
        return ret

    setupfile = os.path.join(tmpdir, "setup.exe")
    if not os.path.exists(setupfile):
        logging.error("Fatal Error. No such file: %s", setupfile)
        return 1


    iss_folder = os.path.join(curdir, "iss")
    # get latest iss template file by modified time
    iss_files = glob.glob(iss_folder+"/*.iss")
    iss_tempalate = max(iss_files, key=os.path.getctime)

    # create iar.iss in tmp dir
    content = None
    tmp_issfile = os.path.dirname(setupfile).replace("\\", "/")+"/iar.iss"

    with open(iss_tempalate, 'r') as fileobj:
        content = fileobj.read()

    with open(tmp_issfile, "w") as issfileobj:
        issfileobj.write(content)

    logging.debug(setupfile)
    logging.debug(tmp_issfile)

    return os.system("start /wait {0} /s /sms /f1{1}".format(setupfile, tmp_issfile))






import os
import glob
import platform

from xml.etree import ElementTree as ET

from mcutk.apps.decorators import build
from mcutk.apps.idebase import IDEBase, BuildResult
from mcutk.apps.eclipse import generate_build_cmdline
from mcutk.debugger.redlink import RedLink




class APP(IDEBase):
    """LPCxpresso IDE for pymcutk."""
    EXITCODE = {
        0 : 'PASS',
        1 : 'APP Error',
        2 : 'Internal Errors' ,
        3 : 'Errors',
        4 : 'Warnings'
    }


    OSLIST = ['Windows', "Linux", "Darwin"]


    def __init__(self, path, *args, **kwargs):
        super(APP, self).__init__("lpcx", path, *args, **kwargs)
        self.builder = self.__builder_init()



    def __builder_init(self):
        """set self.builder value"""
        osname = platform.system()
        builder = ""
        if osname == "Windows":
            builder =  os.path.join(self._path, "ide", "mcuxpressoidec.exe").replace("\\", "/")
        elif osname == "Linux":
            builder = os.path.join(self._path , "ide", "mcuxpressoide")
        elif osname == "Darwin":
            builder = os.path.join(self._path , "ide", "MCUXpressoIDE.app", "Contents", "MacOS", "mcuxpressoide")

        if not os.path.exists(builder):
            if osname == "Windows":
                builder = os.path.join(self._path, "ide", "lpcxpressoidec.exe").replace("\\", "/")
            elif osname == "Linux":
                builder = os.path.join(self._path , "ide", "lpcxpressoide")
            elif osname == "Darwin":
                builder = os.path.join(self._path , "ide", "MCUXpressoIDE.app", "Contents", "MacOS", "lpcxpressoide")

        return builder


    @property
    def is_ready(self):
        return os.path.exists(self.builder)



    @build
    def build_project(self, project, target, logfile, **kwagrs):
        """Return a string about the build command line.

        Arguments:
            project {lpcx.Project} -- lpcx project object
            target {str} -- target name
            logfile {str} -- log file path
            workspace {str} -- default is ~/lpcx_mcutk

        Returns:
            string -- build commandline
        """
        workspace = kwagrs.get("workspace")
        # self.builder = self.builder.replace('\\', '\\\\').replace('/', '\\\\')
        buildcmd = generate_build_cmdline(self.builder, workspace, project.prjdir, project.name, target)

        if logfile:
            buildcmd.append('>> "{}" 2>&1'.format(logfile))

        return " ".join(buildcmd)




    @staticmethod
    def default_install_directory():
        """
        get default installation directory for mcux
        """
        installation_table = {
            "Windows": "C:/nxp/",
            "Linux": "/usr/local/",
            "Darwin": "/Applications",
        }
        osname = platform.system()
        return installation_table[osname]



    @staticmethod
    def get_latest():
        """Search and return a latest IAR instance in system

        Returns:
            <iar.APP object>
        """
        version, path = get_latest()
        if path:
            return APP(path, version=version)
        else:
            return None



    @staticmethod
    def verify(path):
        """
        check the mcux installation path if it is valid!
        """
        osname = platform.system()

        if osname == "Windows":
            return  os.path.join(path, "ide", "mcuxpressoidec.exe").replace("\\", "/")

        elif osname == "Linux":
            return os.path.join(path , "ide", "mcuxpressoide")

        elif osname == "Darwin":
            return os.path.join(path , "ide", "MCUXpressoIDE.app", "Contents", "MacOS", "mcuxpressoide")








    @staticmethod
    def parse_build_result(exitcode, logfile=None):
        """Parse mcuxpressoide build result

        Arguments:
            exitcode {int} -- the exit code of mcuxpressoide process
            logfile {string} -- a file contains the output from mcuxpressoide.

        Returns:
            <BuildResult object> -- the build result

        Notes:
        ---------------------
        mcuxpressoide exit code
            noerror warnnings: 0
            erros: none-zero

        Parser the log file by search the keyword: "Total number of warnings: <>".
        """
        status = APP.EXITCODE.get(exitcode, "Errors")
        return BuildResult.map(status)














def get_latest():
    """Find the latest installation of mcux in system.

    Windows:
        Installed at: in C:/nxp
    Linux:
        Installed at: /usr/local/
        Use Following command:
        >>> dpkg -L mcuxpressoide | grep /usr/local/mcuxpressoide | head -1
        >>> /usr/local/mcuxpressoide-1.0.0.277

    MAC:
        Installed at: /Applications
    """
    osname = platform.system()
    path, version = None, None

    if osname == "Windows":
        parent_dir = APP.default_install_directory()
        installations_list = [dir for dir in os.listdir(parent_dir) if "MCUXpressoIDE" in dir]
        if not installations_list:
            return None, None

        installations_list.sort()
        foldername = installations_list[-1]
        version = foldername.replace("MCUXpressoIDE_", "")
        path = parent_dir+foldername
        return version, path


    #==============================================================
    elif osname == "Linux":
        dirlist = os.listdir("/usr/local/")
        verlist = [edir for edir in dirlist if "mcuxpressoide" in edir]
        maxversion = max(verlist)
        version = maxversion.replace("mcuxpressoide-", "")
        path = "/usr/local/" + maxversion
        return version, path


    #==============================================================
    elif osname == "Darwin":
        dirlist = os.listdir("/Applications")
        mcux_pool = [dirname for dirname in dirlist if "MCUXpressoIDE" in dirname]
        if not mcux_pool:
            return None, None

        mcux_pool.sort()
        latest_installation_dirname = mcux_pool[-1]
        path = "/Applications/%s"%latest_installation_dirname
        version = latest_installation_dirname.replace("MCUXpressoIDE_", "")
        return version, path



import os
import glob

from xml.etree import cElementTree as ET
from mcutk.apps.projectbase import ProjectBase
from mcutk.exceptions import ProjectNotFound
from mcutk.apps import eclipse

class Project(eclipse.Project):
    """ARM DS5 project object

    This class could parser the settings in .cproject and .project.
    Parameters:
        prjpath: path of .project
    """

    PROJECT_EXTENSION = '.cproject'

    def __init__(self, prjpath, *args, **kwargs):
        super(Project, self).__init__(prjpath, **kwargs)
        self.parse(self.prjpath)








import serial
import pexpect
import sys
import time

sys.path.append("./../")

from mcutk.pserial import Serial

ser = Serial('COM20', 9600, timeout=0.1)
spawn = ser.Spawn()

spawn.logfile_read = sys.stdout
spawn.sendline("version\r\n")
spawn.expect([UBOOT_EXPERT, pexpect.TIMEOUT])
logging.info(spawn.before)
spawn.sendline("res\r\n")
spawn.expect(['Hit any key to stop autoboot:', pexpect.TIMEOUT])
time.sleep(1)
spawn.sendline("a")
spawn.expect([UBOOT_EXPERT, pexpect.TIMEOUT])
logging.info(spawn.before)
serobj.close()




spawn.write("a")


try:
    spawn.expect("welcome", timeout=3)
except:
    return

spawn.write("a")
try:
    spawn.expect("welcome", timeout=3)
except:
    return

spawn.write("a")
try:
    spawn.expect("welcome", timeout=3)
except:
    return

spawn.expect([pexpect.EOF, pexpect.TIMEOUT])
spawn.close()






"""
Project interface definition.
"""
import abc
import os
import glob

from mcutk.exceptions import ProjectNotFound, InvalidTarget



class ProjectBase(object):
    """
    abstract class representing a project basic function
    """
    __metaclass__ = abc.ABCMeta

    # project extension
    PROJECT_EXTENSION = None

    @classmethod
    def frompath(cls, path):
        """Return a project instance from a given file path or directory.

        If path is a directory, it will search the project file and return an instance.
        Else this will raise mcutk.apps.exceptions.ProjectNotFound.
        """
        if os.path.isfile(path):
            return cls(path)

        if not cls.PROJECT_EXTENSION:
            raise ValueError("Error, %s.PROJECT_EXTENSION is not defined! Please report an bug!"%cls)

        try:
            project_file = glob.glob(path + "/" + cls.PROJECT_EXTENSION)[0]
        except IndexError:
            raise ProjectNotFound("Not found %s in specific folder"%cls.PROJECT_EXTENSION)

        return cls(project_file)



    def __init__(self, path, *args, **kwargs):
        self.prjpath = path
        self.prjdir = os.path.dirname(path)
        self._conf = None
        self._targets = list()


    @property
    def targets(self):
        """Get targets"""
        return list(self._targets)


    @property
    def targetsinfo(self):
        """Returns a dict about the targets data.

        Example:
        {
            "Debug":   "debug_output_dir/output_name",
            "Release": "release_output_dir/output_name",
        }
        """
        return self._conf


    @abc.abstractproperty
    def name(self):
        """Get project name"""
        pass


    def map_target(self, input_target):
        input_target = input_target.strip()
        # map for mdk
        for tname in self.targets:
            if input_target == tname or input_target in tname.split() :
                return tname

        # map for general
        for tname in self.targets:
            if input_target.lower() in tname.lower():
                return tname

        for tname in self.targets:
            print("!@ avaliable target: %s"%tname)

        msg = "Cannot map the input target: {}, project: {}, valid targets: {}".format(input_target, self.prjpath, str(self.targets))
        raise InvalidTarget(msg)



    @property
    def idename(self):
        return str(self.__module__).split('.')[-2]


    def to_dict(self):
        return {
            'toolchain': self.idename,
            'targets': self.targets,
            'project': self.prjpath,
            'name': self.name
        }





import os
import glob
import platform
import logging
import subprocess
try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser

from .app import APP, get_latest
from mcutk.cmsispack import install_pack_unzip
from mcutk.apps.decorators import appinstall


@appinstall
def install(installer, license, install_path='C:\Keil_v5'):
    """Install uv4/mdk.

    Arguments:
        installer {string} -- path to installer
        license {[type]} -- network license
        install_path {string} -- path to install location, default is "C:\Keil_v5"

    Returns:
        tuple -- (path, version)
    """
    try:
        _silent_install(installer, license, install_path)
    except subprocess.CalledProcessError:
        logging.error("failed to install %s"%installer)
        return False

    # install the packs under ./ARM/PACK/.Download
    uv4 = APP(install_path, "v")
    prepack = os.path.join(install_path, "ARM/PACK/.Download")
    packages = glob.glob(prepack+"/*.pack")
    for pack in packages:
        uv4.install_pack(pack)

    path, version = get_latest()

    return path, version





def _silent_install(installer, license, install_path='C:\Keil_v5'):
    """silent install MDK

    Arguments:
        installer {string} -- path to installer
        license {string} -- network license
        install_path {string} -- path to install location, default is "C:\Keil_v5"

    Raise Exceptions:
        When the unzip unsuccessful, it will raise an Error

    Automated installation:
        This only support windows, there are three steps in the progress:
            1. unzip installer file.
            2. generate TOOLS.ini by use the template SETUP.ini.
            3. install .pack under ARM/PACK/.DOWNLOADS
    """
    logging.info("install path: %s", install_path)

    uv4_ini = {
        'RTEPATH': install_path + r"\ARM\PACK",
        'TOOL_VARIANT': 'mdk_pro_flex',
        'FLEX': license,
        'FLEX_USE': '1',
        'FLEX_VARIANT':'mdk_pro',
        'LEGACY_CM': '1',
        'ORGANIZATION': "NXP",
        'NAME': '"Test", "Team"',
        'EMAIL': "dapeng@nxp.com"
    }

    osname = platform.system()
    if osname != "Windows":
        raise SystemError("Windows only")

    curdir = os.path.dirname(os.path.abspath(__file__)).replace("\\", "/")
    unzip_tool = os.path.abspath(curdir + "../../bin/7z.exe")

    # unzip uv4 installer.
    subprocess.check_call("{0} x -o{1} -y {2}".format(unzip_tool, install_path, installer))

    # Create and configure the TOOLS.ini from SETUP.ini template
    cf = ConfigParser.ConfigParser()
    cf.read(install_path+"/SETUP.ini")
    for key, v in uv4_ini.items():
        print("set {0} {1}".format(key, v))
        cf.set("UV2", key, v)
    cf.set("ARMADS", "PATH", install_path + "\ARM\\")
    cf.write(open(install_path+"/TOOLS.ini", "w"))

    return 0



import os
import re
import glob
import time
import zipfile
import logging
import platform
import subprocess
import os.path as Path
try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser

from mcutk.apps.decorators import build
from mcutk.apps.idebase import IDEBase, BuildResult
from mcutk.apps.mdk.project import Project
from mcutk.util import copydir, run_command


class APP(IDEBase):
    """This class wrap ARM-MDK to provide ease-to-use interface for outside.

    """
    OSLIST = ['Windows']


    EXITCODE = {
        0:	"PASS",
        1:	"Warnings",
        2:	"Errors",
        3:	"Fatal Errors",
        11:	"Cannot open project file for writing",
        12:	"Device with given name in not found in database",
        13:	"Error writing project file",
        15:	"Error reading import XML file",
    }


    @staticmethod
    def get_latest():
        """Return an instance by latest version"""

        if os.name == 'nt':
            path, version = get_latest()
            if path:
                return APP(path, version=version)
        return None




    @staticmethod
    def parse_build_result(exitcode, logfile=None):
        """Parse the build result form exitcode & logfile

        Arguments:
            exitcode {int} -- build process exit code
            logfile {int} -- build log file

        Returns:
            BuildResult object
        """
        status = APP.EXITCODE.get(exitcode, "Errors")
        return BuildResult.map(status)



    @staticmethod
    def verify(path):
        osname = platform.system()
        if osname == "Windows":
            return os.path.exists(path + "/UV4/UV4.exe")

        elif osname in ["Linux", "Darwin"]:
            raise OSError("Not Defined")




    def __init__(self, *args, **kwargs):
        super(APP, self).__init__("mdk", *args, **kwargs)
        self.builder = os.path.join(self.path, "UV4", "UV4.exe").replace('\\', '/')




    @property
    def is_ready(self):
        return os.path.exists(self.builder)



    @build
    def build_project(self, project, target, logfile=None, **kwargs):
        """
        Generate build commands.

        Arguments:
            project: uv4 Project object.
            target: target name.
            logfile: path to log file.

        Returns:
            string -- build commands line

        """
        target = project.map_target(target)
        project.set_build_options()
        buildcmd = '{0} -r {1} -q -j0 -t "{2}"'.format(
            self.builder,
            project.prjpath,
            target)

        if logfile:
            buildcmd += ' -o %s'%logfile
        return buildcmd



    # def create_project(self):
    #     buildcmd = '{0} -r "{1}" -j0 -o "{2}" -t "{3}"'.format(
    #         self.builder,
    #         projpath, buildlog, target)




    def install_pack(self, packpath):
        """Install cmsis pack by PackUnzip.exe
        """
        packTool = self.path + "/PackUnzip.exe"
        if os.path.exists(packTool) is False:
            raise IOError("No such file: %s"%packTool)
        ret = os.system("{0} -i {1} --skip-check --no-gui".format(packTool, packpath))
        if ret:
            print("Exit Code: %s"%ret)
            os._exit(ret)
        else:
            print("Successfully")





    def enable_license(self, license):
        """Enable uv4 flex license."""
        cf = ConfigParser.ConfigParser()
        tools_ini = os.path.dirname(self.path)+"/TOOLS.ini"
        cf.read(tools_ini)
        cf.set("UV2", "FLEX", license)
        cf.write(open(tools_ini, "w"))



    def transform_elf(self, type, in_file, out_file):
        """Call <keil_root>/ARM/ARMCC/bin/fromelf.exe."""

        supported_types = {
            'bin': '--bin',
            'srec': '--m32combined',
            'ihex': '--i32',
        }

        if type not in supported_types:
            raise ValueError('unsupported type')

        type_option = supported_types.get(type)

        fromelf_tool = Path.join(self.path, 'ARM/ARMCC/bin/fromelf.exe')
        fromelf_command = [
            fromelf_tool,
            type_option,
            '--output',
            out_file,
            in_file
        ]
        return subprocess.call(fromelf_command) == 0






    def programming(self, board, prjdir, target, file, action='program', **kwargs):
        """MDK programming.

        Arguments:
            prjdir {str} -- project directory
            usbid {str} -- usbid
            target {str} -- app target
            file {str} -- file path
            action {str}  -- program/erase
            timeout {int}  -- timeout for action
        Returns:
            Tuple -- (returncode, output)
        """
        timeout = kwargs.get('timeout', 120)
        # is_to_flash = kwargs.get('flash', True)
        project = Project.frompath(prjdir)

        if action == "erase":
            for t in project.targets:
                if 'flexspi_nor' in t:
                    target = t
                    break

        target = project.map_target(target)
        is_debug_mode = project.set_debugging_options(target, sn=board.usbid, debugfile=file, action=action)

        if not is_debug_mode:
            cmd = '{0} -f {1} -t "{2}"'.format(self.builder, project.prjpath, target)
        else:
            cmd = '{0} -d {1} -t "{2}"'.format(self.builder, project.prjpath, target)
        return run_command(cmd, timeout=timeout)




def get_latest():
    osname = platform.system()
    if osname == "Windows":
        try:
            import _winreg as winreg
        except ImportError:
            import winreg

        root_key_path = r"SOFTWARE\Wow6432Node\Keil\Products\MDK"
        try:
            key_object = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, root_key_path, 0, winreg.KEY_READ)
            path = winreg.QueryValueEx(key_object, "Path")[0].replace('\\', '/')
            keilroot = os.path.dirname(path)
            path = keilroot
            version = winreg.QueryValueEx(key_object, "Version")[0]
            winreg.CloseKey(key_object)

            cf = ConfigParser.ConfigParser()
            cf.read(keilroot+"/TOOLS.ini")
            ver = cf.get("ARM", "VERSION")
            if ver not in version:
                logging.debug("Registry record is: %s, but actually it is: %s", version, ver)

            return path, ver

        except Exception as e:
            logging.warning("Not found Keil installation infomation in windows register")


        keilroot = "C:/Keil_v5"
        tools_ini = keilroot+"/TOOLS.ini"
        if not os.path.exists(tools_ini):
            return None, None

        cf = ConfigParser.ConfigParser()
        cf.read(tools_ini)
        version = cf.get("ARM", "VERSION")
        path = keilroot

        return path, version

    else:
        raise OSError("ARM MDK(Keil) is unavaliable on such os!")




"""
This example show how to use mcutk.apps to project.
"""
import os
import sys
import subprocess
import argparse

sys.path.append("./../")
from mcutk.apps import factory


parser = argparse.ArgumentParser()
parser.add_argument("-n", dest="name", help="app name")
parser.add_argument("-p", dest="project", help="project path")
args = parser.parse_args(sys.argv[1:])

# get app module
appmodule = factory(args.name)

# get instance by scaning the system
app = appmodule.APP.get_latest()

print app

if args.project:
    print args.project
    project = appmodule.Project(args.project)
    for tname in project.targets:
        logfile = os.path.join(os.path.dirname(args.project), tname + "_log.log")
        print "%s is building..."%tname
        print app.build_project(project, tname, logfile)







"""
IDE base class definition.
"""
import abc
import os

from enum import Enum
from mcutk.appbase import APPBase
from mcutk.elftool import transform_elf_basic

# default mcutk workspace for eclipse project
DEFAULT_MCUTK_WORKSPACE = os.path.expanduser('~') + '/mcutk_workspace'



class Result(Enum):
    Pass = 0
    Errors = 1
    Warnings = 2
    OtherErrors = 3
    Timeout = 4


class BuildResult(object):
    """The class represent build result object."""

    def __init__(self, result, output):
        self._result = result
        self._output = output


    @property
    def result(self):
        return self._result


    @property
    def output(self):
        return self._output

    @property
    def name(self):
        return self._result.name


    @property
    def value(self):
        return self._result.value


    def set_output(self, v):
        self._output = v



    @classmethod
    def map(cls, result, output=None):
        value = result.lower()
        if value in ("pass", "success", "true", "noerror"):
            ret = Result.Pass

        elif value in ("warnings", "warning"):
            ret = Result.Warnings

        elif value in ("errors", 'error'):
            ret = Result.Errors

        elif value in ('timeout'):
            ret = Result.Timeout

        else:
            ret = Result.OtherErrors

        return cls(ret, output)



class IDEBase(APPBase):
    """
    An abstract class representing an ide.
    """
    __metaclass__ = abc.ABCMeta

    ISIDE = True



    @staticmethod
    def parse_build_result():
        """Parse the build result: warnning? pass? failed?"""
        raise NotImplementedError()



    def __init__(self, *args, **kwargs):
        super(IDEBase, self).__init__(*args, **kwargs)




    @abc.abstractmethod
    def build_project(self, project, target, logfile):
        """Return a string about the build command line."""
        pass


    def __str__(self):
        return self.name + "-" + self.version



    def transform_elf(self, type, in_file, out_file):
        """ELF file format converter.
        This is a general method for general ide instance.
        It will calle mcutk/bin/arm-none-eabi-objcopy to do the converter.

        Supported types: bin, ihex, srec.

        Arguments:
            type {str} -- which type you want to convert.
            in_file {str} -- path to elf file.
            out_file {str} -- output file

        Raises:
            ReadElfError -- Unknown elf format will raise such error
            Exception -- Convert failed will raise exception

        Returns:
            bool
        """
        if os.name == 'nt':
            return transform_elf_basic(type, in_file, out_file)
        else:
            logging.error("no implement")
            return False







from __future__ import print_function
import os
import sys
import re
import platform
import subprocess
import logging
import threading
import time
import tempfile
import shlex
import glob
import pexpect
# import psutil
import socket
import telnetlib

from xml.etree import ElementTree as ET
from threading import Thread, Timer
from pexpect.popen_spawn import PopenSpawn
from mcutk.debugger.base import DebuggerBase
from mcutk.util import run_command
from mcutk.gdb_session import GDBSession
from mcutk.debugger.base import _validate_port_is_ready

class RedLinkServerIssue(Exception):
    pass

class RedLink(DebuggerBase):
    """
    Redlink (also known as Link Server) is the mechanism used by LPCXpresso to provide a debug
    connection down to a target MCU. It provides the means to connect to single MCUs or to multiple
    CPUs within the same MCU via a single debug probe.

    Command line flash programming reference:
        https://community.nxp.com/thread/389139


    Notes from MCUXpresso IDE User's Guide:
        The MCUXpresso IDE Flash programming utility is located at:
        - crt_emu_cm_redlink.exe is the name of the Flash utility, it is located at <mcuxpressod_root>/ide/bin/.
        - target is the target chip name. For example LPC1343, LPC1114/301, LPC1768 etc. (see
        'Finding Correct Parameters...' below)
        - --flash-load can actually be one of a few different options. Use:
        - --flash-load to write the file to Flash,
        - --flash-load-exec to write it to Flash and then cause it to start running,
        - --flash-mass-load to erase the Flash and then write the file to the Flash, and
        - --flash-mass-load-exec to erase the Flash, write the file to Flash and then cause it to start running.
        - filename is the file to Flash program. It may be an executable (axf) or a binary (bin) file. If using
        a binary file, the base_address also must be specified. Using enclosing quotes is optional
        unless the name includes unusual characters or spaces.
        - base_address is the address where the binary file will be written. It can be specified as a hex
            value with a leading 0x.

        If you are using Flash memory that is external to the main chip you will need to specify an
        appropriate Flash driver that supports the device. This usually takes the name of a .cfx file held
        in a default location. In unusual circumstances it is possible to specify an absolute file system
        name of a file. Using enclosing quotes is optional unless.
    """
    def __init__(self, *args, **kwargs):
        super(RedLink, self).__init__("redlink", *args, **kwargs)

        # MCUXpressoIDE 10.2 & 10.1
        mcux_tool_bin = os.path.join(self.path, "bin")
        self._is_old_version = True

        # For MCUXpressoIDE 10.3 & later
        if not os.path.exists(mcux_tool_bin):
            plugins_dir = os.path.join(self.path, "plugins")
            try:
                tools_bin = glob.glob(plugins_dir+"/com.nxp.mcuxpresso.tools.bin.*")[0]
                mcux_tool_bin = os.path.join(tools_bin, "binaries")
                self._is_old_version = False
            except IndexError:
                logging.warning('cannot found redlinkserv.exe, %s', plugins_dir)

        self._crt_emu_redlink = os.path.join(mcux_tool_bin, "crt_emu_cm_redlink.exe").replace("\\", "/")
        self._redlinkserver = os.path.join(mcux_tool_bin, "redlinkserv.exe").replace("\\", "/")
        self.template_root = None
        self.tn = None
        self._redlinkserverpro = None


    @property
    def is_ready(self):
        return os.path.exists(self._crt_emu_redlink)


    def set_board(self, board):
        # workaround to clear the prefix of usbid
        if ":" in board.usbid:
            board.usbid = board.usbid.split(":")[-1]
        self._board = board


    def erase(self):
        """
        Erase flash by: -flash-erase
        """
        logging.info("erase flash")
        crt_redlink_cmd = self.gen_crt_cmd(self._board) + " -flash-erase "
        return run_command(crt_redlink_cmd, timeout=120)




    def reset(self):
        """
        reset board by: pyocd-tool.exe reset -b <id>
        """
        logging.warning("cannot reset board by redlink")
        return False




    def _start_linkserver(self, connectscript=None):
        # start linkserver in background
        args = [
            self._redlinkserver,
            "--telnetport=%s"%self._board.gdbport,
            "--port=%s"%(int(self._board.gdbport) + 30)
        ]

        if connectscript:
            args.append("--connectscript %s"%connectscript.replace('\\', '/'))

        self._redlinkserverpro = subprocess.Popen(args)





    def _get_probe_index(self, usbid, retry=3):
        pattern = re.compile("serial number = .*\n", re.I)
        usbid_list = []

        for _ in xrange(retry):
            try:
                if not(self._redlinkserverpro and self._redlinkserverpro.poll() is None):
                    raise RedLinkServerIssue('Server is not running')

                self.tn = telnetlib.Telnet('localhost', self._board.gdbport, timeout=10)
                logging.info('connected redlinkserver: %s', self._board.gdbport)
                time.sleep(0.5)
                self.tn.write('PROBELIST\n')
                output = self.tn.read_until('redlink>PROBELIST', timeout=3)
                usbid_list = pattern.findall(output)
                usbid_list = [uid.split(" = ")[-1].replace("\n", "") for uid in usbid_list]
                if usbid_list:
                    break
                else:
                    logging.error('usbid list is empty')

            except Exception as e:
                logging.exception('failed to get probeindex, retry again.')
                time.sleep(0.5)
        else:
            raise RedLinkServerIssue('server is not ready')


        try:
            index = usbid_list.index(usbid)
        except ValueError:
            logging.error(usbid_list)
            raise RedLinkServerIssue("Usb SN '{0}' is not connect!".format(usbid))

        index += 1
        print("UsbID: %s, Probe Index: %s"%(usbid, index))

        return index







    def gen_crt_cmd(self, board, connectscript=None, resetscript=None, device_parts_dir=None, flash_driver_dir="default", coreindex=0):
        """Generate crt_emu_redlink command line string."""
        conf = _parse_conf_from_template(self.template_root)
        conf['connectscript'] = None

        # There is a bug in crt_emu_redlink.exe, when the connscript is set, the client will always use the probeindex 1
        # if connectscript and os.path.exists(connectscript):
        #     conf['connectscript'] = connectscript

        if conf['devicename']:
            board.devicename = conf['devicename']

        _params = [
            self._crt_emu_redlink,
            "-p {}".format(board.devicename),
            "-g",
            # "--cache disable",
            # "--no-info-roms",
            # "--no-packed",
            # "--reset system,soft",
            "--flash-driver= ",
            "-COREINDEX=%s"%coreindex
            # "--telnet 3333",
        ]

        # workaround for MIMXRT1052xxxxB, https://jira.sw.nxp.com/browse/RE-1830
        if board.devicename in ['MIMXRT1052xxxxB', 'MIMXRT1021xxxxx']:
            _params.append('--reset VECTRESET')

        # self._start_linkserver(connectscript)

        # For 10.2 version, we need to get the probeindex at first.
        # New in MCUXpresso IDE Version 10.3 eb2 is the option to reference a debug probe via its serial number from the
        # command line. This feature allows multiple debug probes to be connected (over USB) at the same time and referenced
        #  individually. Currently this feature is not used directly by the IDE but this may change before we release
        # version 10.3.
        if self._is_old_version:
            probe_index = self._get_probe_index(board.usbid)
            _params.append("-PROBEHANDLE=%s"%probe_index)
        else:
            _params.append("--probeserial=%s"%board.usbid)


        if conf['connectscript']:
            _params.append("--connectscript %s"%conf['connectscript'].replace('\\', '/'))

        if conf['device_parts_dir']:
            _params.append("-x {}".format(conf['device_parts_dir'].replace('\\', '/')))

        # don't use flash driver for RAM and SDRAM
        if flash_driver_dir is None:
            _params.append("--no-flash-dir-default")
        else:
            # add falsh_dir
            if flash_driver_dir not in ('default', ''):
                _params.append("--flash-dir {}".format(flash_driver_dir.replace('\\', '/')))
            # add falsh_dir from configuration
            if conf['flash_driver_dir']:
                _params.append("--flash-dir {}".format(conf['flash_driver_dir'].replace('\\', '/')))

        if resetscript:
            _params.append("--resetscript {}".format(resetscript.replace('\\', '/')))

        return " ".join(_params)




    def gdb_init_template(self):
        """Default gdb commands for crt_emu_redlink.
        Reference from https://community.nxp.com/inbox.
        mon ondisconnect <type>
            type: Must be one of: nochange, stop, cont, run_cont.
        """
        commands = \
'''set non-stop on
set remotetimeout 100000
target extended-remote | {}
set mem inaccessible-by-default off
mon semihosting dis
mon ondisconnect cont
set arm force-mode thumb
load
q
'''
        return commands





    def _crt_debugging(self, filepath, crt_redlink_cmd, timeout):
        # add flash operation
        crt_redlink_cmd += ' -flash-load-exec "{}"'.format(filepath.replace("\\", "/"))
        # add start address
        if filepath.endswith('.bin'):
            if self._board.start_address:
                crt_redlink_cmd += " --load-base=%s"%self._board.start_address
            # else:
            #     # default value
            #     crt_redlink_cmd += " --load-base=0x0"

        # call registerd callback function
        self._call_registered_callback("before-load")

        logging.info(crt_redlink_cmd)

        ret, console = run_command(crt_redlink_cmd, timeout=timeout)

        logging.info('linkserver exit code: %s', ret)

        # workaround: force kill alive redlinkserv process
        if ret != 0:
            os.system('''TASKKILL /F /T /FI "IMAGENAME eq redlinkserv*"''')

        return ret, console




    def _gdb_debugging(self, filename, gdbinit, timeout):
        if not self.gdbpath or not os.path.exists(self.gdbpath):
            raise IOError('gdb executable is not exists!')

        def _check_gdb_cmd_run_error(text):
            return 'failed on connect' in text or 'load failed' in text \
                or "error finishing flash operation" in text \
                or 'redlink interface error' in text

        gdb_cmd = "{} --exec {} --silent".format(self.gdbpath, filename)
        session = GDBSession.start(gdb_cmd)
        errorcode = 0

        # convert string commands to a list
        _gdb_actions = [line.strip() for line in gdbinit.split("\n") if line.strip()]

        for act in _gdb_actions:
            # call registerd callback function before-load command
            if act == 'load':
                self._call_registered_callback("before-load")
            try:
                c = session.run_cmd(act).lower()
                if _check_gdb_cmd_run_error(c):
                    errorcode = 1
                    break
            except:
                logging.exception('gdb cmd run exception, CMD: %s', act)

        session.close()

        return errorcode, session.console_output



    def flash(self, filepath, connectscript=None, resetscript=None, device_parts_dir=None, flash_driver_dir="default", **kwargs):
        """MCUXPresso IDE crt_emu_cm_redlink utility interface.

        This will program image by crt_emu_cm_redlink tool.

        Arguments:
            filepath {str} -- path to image file

        Keyword Arguments:
            base_addr {int} -- the base address to programming(default: 0)
            usbid {str}  -- link2/cmsis-dap usb serial number.
            connectscript {str} -- connect script (default: None)
            resetscript {str} -- reset script (default: None)
            device_parts_dir {str} -- directory include devices details(default: {None})
            flash_driver_dir {str} -- directory include flash drivers(default: {None})

        Raises:
            ValueError -- [description]

        Returns:
            Tuple -- (returncode, output)
        """
        crt_redlink_cmd = self.gen_crt_cmd(
            self._board,
            connectscript,
            resetscript,
            device_parts_dir,
            flash_driver_dir,
            0)

        timeout = 300
        ret = self._crt_debugging(filepath, crt_redlink_cmd, timeout)
        # if filepath.endswith('.bin'):
            # ret = self._crt_debugging(filepath, crt_redlink_cmd, timeout)
        # else:
            # gdb_init = self.gdb_init_template().format(crt_redlink_cmd)
            # ret = self._gdb_debugging(filepath, gdbinit=gdb_init, timeout=timeout)

        if self.tn:
            self.tn.close()

        return ret




def _parse_conf_from_template(path):
    ret = {
        "connectscript": None,
        "devicename": None,
        "device_parts_dir": None,
        "flash_driver_dir": None,
    }

    if not path:
        return ret

    if not os.path.exists(path):
        return ret

    ret['device_parts_dir'] = path
    ret['flash_driver_dir'] = path
    flash_dir = os.path.join(path, 'Flash')
    if os.path.exists(flash_dir):
        ret['flash_driver_dir'] = flash_dir


    try:
        project_conf_xml = glob.glob(os.path.join(path, "com.crt.advproject.config.exe.*.xml"))[0]
        root = ET.parse(project_conf_xml).getroot()
        node = root.find("parameters/initvalue[@var='internal.connect.script']")
        if node is not None:
            ret['connectscript'] = node.attrib['value']
    except IndexError:
        pass
    except Exception as e:
        raise e

    try:
        device_part_xml = glob.glob(os.path.join(path, "*_dir_part.xml"))[0]
        root = ET.parse(device_part_xml).getroot()
        node = root.find("chip")
        if node is not None:
            devicename = node.attrib['name']
            ret['devicename'] = devicename
            logging.info("Found Device Name: %s", devicename)
    except IndexError:
        pass

    return ret






from mcutk.exceptions import (ProjectNotFound, ProjectParserError, InstallError)

__all__ = ["appfactory", "factory"]



def appfactory(name):
    """Return specific APP class.

    Example 1, basic:
        >>> APP = appfactory('iar')
        <mcutk.apps.iar.APP object at 0x1023203>

    Example 2, get the latest instance by scanning your system:
        >>> app = appfactory('iar').get_latest()
        >>> print app.path
        C:/program files(x86)/IAR Systems/IAR Workbench/
        >>> print app.version
        8.22.2

    Example 3, create app instance directly:
        >>> APP = appfactory('iar')
        >>> app = APP('/path/to/ide', version='1.0.0')
        >>> print app.path
        C:/program files(x86)/IAR Systems/IAR Workbench/
        >>> print app.version
        8.22.2

    Example 4, load and parse the project:
        >>> project = appfactory('iar').Project('/path/to/project')
        >>> print project.name
        hello_world
        >>> print project.targets
        ['debug', 'release']

    """
    import importlib
    try:
        idemodule = importlib.import_module("mcutk.apps.%s"%(name))
        appcls = getattr(idemodule, "APP")
        projcls = getattr(idemodule, "Project")
        appcls.Project = projcls
    except ImportError:
        return None

    return appcls



def factory(name):
    """Return specific app module"""
    import importlib
    try:
        idemodule = importlib.import_module("mcutk.apps.%s"%(name))
    except ImportError:
        pass
    return idemodule



from setuptools import setup, find_packages
import os
import sys
import subprocess

version = '0.1.2'

install_requires = [
    'pyocd',
    'pyserial>=3.4',
    'xmodem',
    'future',
    "pexpect",
    "click",
    "pyelftools",
    "pyyaml",
    'globster',
    'enum34'
]

version_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'mcutk/_version.py')

if not os.path.exists(version_file):
    with open(version_file, 'w') as f:
        f.write("VERSION='%s'"%version)


setup(
    name="pymcutk",
    version=version,
    url='https://github.com/Hoohaha/pymcutk',
    description="MCU toolkit for mcu automated testing.",
    long_description=str(open('README.md', 'rb').read()),
    author="Haley Guo, Fly Yu",
    license="MIT License",
    author_email="hui.guo@nxp.com, chuzhi.yu@nxp.com",
    install_requires=install_requires,
    packages=["mcutk"],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'mtk = mcutk.__main__:main',
        ]
    },
    setup_requires=[
        'setuptools',
        'setuptools-git',
        'wheel',
    ],
    classifiers = [
        'Development Status :: 4 - Beta',
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
)


import os
import sys
import click
import glob
import logging
from os.path import join

from mcutk.apps import appfactory
from mcutk.board import Board
from mcutk.managers.conf_mgr import ConfMgr
from mcutk.managers.debug_helper import find_debuggers, list_devices, select_device





@click.command('flash', short_help='flash debug file to board')
@click.argument('path', required=True, type=click.Path(exists=True))
@click.option('-a', '--base-address', default=0, help='base address to load.')
@click.option('-c', '--config', help='load configuration from file.')
def cli(path, base_address, config):
    gdb = None

    # get gdb
    armgcc = appfactory('armgcc').get_latest()
    if armgcc and armgcc.is_ready:
        gdb = os.path.join(armgcc.path, 'bin/arm-none-eabi-gdb')

    all_debuggers = find_debuggers()
    devices = list_devices(all_debuggers)
    selected_debugger, selected_device = select_device(all_debuggers, devices)

    selected_debugger.gdbpath = gdb
    devicename = selected_device.get('devicename')
    board = Board(devicename, **selected_device)
    board.name = selected_device.get('name', 'unknown')
    board.debugger_type = selected_debugger.name
    board.debugger = selected_debugger
    board.start_address = base_address
    board.programming(path)




import os
import shutil
from xml.etree import cElementTree as ET
from mcutk.apps.projectbase import ProjectBase
from mcutk.exceptions import ProjectNotFound



def generate_build_cmdline(executor, workspace, project_root, project_name,
    target='all', application='org.eclipse.cdt.managedbuilder.core.headlessbuild'):
    """Generate and return C/C++ build command line for Eclipse project. Return type list of strings.

    This is a common interface for generatl eclipse project.

    Eclipse runtime documentation:
    https://help.eclipse.org/kepler/index.jsp?topic=%2Forg.eclipse.platform.doc.isv%2Freference%2Fmisc%2Fruntime-options.html&anchor=eclipseproduct


    Note about eclipse command line mode.

    eclipsec --launcher.suppressErrors -nosplash
        -application {org.eclipse.cdt.managedbuilder.core.headlessbuild|other applications}
        [-cleanBuild|-build] {project_name_reg_ex/config_name_reg_ex | all}
        [-import|-importAll] {[uri:/]/path/to/project}}
        -data {workspace path}

        =======================     =======================
        Parameter                   Description
        =======================     =======================
        -application applicationId  The application to run. Applications are declared by plug-ins supplying extensions to the org.eclipse.core.runtime.applications extension point. This argument is
                                    typically not needed. If specified, the value overrides the value supplied by the configuration. If not specified, the Eclipse Workbench is run.
        --launcher.suppressErrors   If specified the executable will not display any error or message dialogs. This is useful if the executable is being used in an unattended situation
        -nosplash                   Controls whether or not the splash screen is shown.
        -data workspacePath         The path of the workspace on which to run the Eclipse platform. The workspace location is also the default location for projects. Relative paths are interpreted.
                                    relative to the directory that Eclipse was started from.
        -nosplash                   Runs the platform without putting up the splash screen.
        -import                     {[uri:/]/path/to/project} -- Import projects under URI.
        -importAll                  {[uri:/]/path/to/projectTreeURI} -- Import all projects under URI.
        -build                      {project_name_reg_ex{/config_reg_ex} | all} -- Build projects.
        -cleanBuild                 {project_name_reg_ex{/config_reg_ex} | all} -- Clean and  build projects.
        -I                          {include_path} -- additional include_path to add to tools
        -include                    {include_file} -- additional include_file to pass to tools
        -D                          {prepoc_define} -- addition preprocessor defines to pass to the tools
        -E                          {var=value} -- replace/add value to environment variable when running all tools
        -Ea                         {var=value} -- append value to environment variable when running all tools
        -Ep                         {var=value} -- prepend value to environment variable when running all tools
        -Er                         {var} -- remove/unset the given environment variable
        -T                          {toolid} {optionid=value} -- replace a tool option value in each configuration built
        -Ta                         {toolid} {optionid=value} -- append to a tool option value in each configuration built
        -Tp                         {toolid} {optionid=value} -- prepend to a tool option value in each configuration built
        -Tr                         {toolid} {optionid=value} -- remove a tool option value in each configuration built
    """
    # Note:
    # If project already in workspace, eclipse will alert an error and exit.
    # To resolve this pain, force remove tree '.metadata\.plugins\org.eclipse.core.resources'
    if os.path.exists(workspace):
        eclipse_core_resources = os.path.join(workspace, '.metadata/.plugins/org.eclipse.core.resources')
        if os.path.exists(eclipse_core_resources):
            shutil.rmtree(eclipse_core_resources, ignore_errors=True)

    project_root = os.path.abspath(project_root)
    return [
        executor,
        "--launcher.suppressErrors",
        "-noSplash",
        "-consoleLog",
        "-application",
        application,
        "-data",
        workspace,
        "-importAll",
        project_root,
        "-cleanBuild",
        "%s/%s"%(project_name, target)
    ]



class Project(ProjectBase):
    """Present Eclipse C/C++ Project object.
    """

    def __init__(self, path, *args, **kwargs):
        super(Project, self).__init__(path, **kwargs)
        self._name = None
        self._targets = dict()
        self._conf = dict()


    def parse_project(self, path):
        xml_root = ET.parse(path).getroot()
        return xml_root.find('./name').text.strip()


    def parse_cproject(self, cpath):
        targets = {}
        xml_root = ET.parse(cpath).getroot()
        for per_node in xml_root.findall('.//configuration[@buildArtefactType]'):
            target_name = per_node.attrib.get('name').strip()
            extension = per_node.attrib.get('artifactExtension', '').strip()

            artifactName = per_node.attrib.get('artifactName').strip()
            if artifactName == "${ProjName}":
                artifactName = self._name

            targets[target_name] = target_name + "/" + artifactName + '.' + extension

        return targets


    def parse(self, path):
        if not path.endswith('.project'):
            path = os.path.join(self.prjdir, '.project')

        cpath = os.path.join(self.prjdir, '.cproject')
        self._name = self.parse_project(path)
        self._conf = self.parse_cproject(cpath)
        self._targets = self._conf.keys()



    @property
    def name(self):
        """Return the application name

        Returns:
            string --- app name
        """
        return self._name



    @property
    def targetsinfo(self):
        """Returns a dict about the targets data.

        Example:
        {
            "Debug":   "debug_output_dir/output_name",
            "Release": "release_output_dir/output_name",
        }
        """
        return self._conf



import os
import glob
from xml.etree import cElementTree as ET
from mcutk.apps.projectbase import ProjectBase


class Project(ProjectBase):
    """
    KDS project object

    This class could parser the settings in .cproject and .project.
    Parameters:
        prjpath: path of .project

    """

    PROJECT_EXTENSION = '.project'

    def __init__(self, prjpath, *args, **kwargs):
        super(Project, self).__init__(prjpath, *args, **kwargs)
        try:
            self.cprjpath = glob.glob(self.prjdir + "/.cproject")[0]
        except Exception:
            raise IOError(".cproject file not found!")

        self._confs = self._get_all_configuration()
        self._targets = self._confs.keys()




    @property
    def name(self):
        """Return the application name

        Returns:
            string --- app name
        """
        xml_root = ET.parse(self.prjpath).getroot()
        app_name = xml_root.find('./name').text.strip()

        return app_name



    def _get_all_configuration(self):
        """read all configuration from .cproject file

        Raises:
            IOError -- if .cproject is not exists, it will raise an IOError.

        Returns:
            dict -- targets configuration
        """
        targets = {}

        xml_root = ET.parse(self.cprjpath).getroot()

        for per_node in xml_root.findall('.//configuration[@buildArtefactType="org.eclipse.cdt.build.core.buildArtefactType.exe"]'):
            target_name = per_node.attrib.get('name').strip()
            output_dir  = target_name
            output_name = self.name + '.' +  per_node.attrib.get('artifactExtension').strip()
            targets[target_name] = (output_dir, output_name)
        return targets








import os
import re
import glob
import logging
import platform
import subprocess

from distutils.version import LooseVersion
from mcutk.apps.decorators import build
from mcutk.apps.idebase import IDEBase, BuildResult
from mcutk.elftool import transform_elf_basic



class APP(IDEBase):
    """GNU ARM GCC compiler.

    CMake and ARM-GCC build explanation:
        - Generate Makefile:
        >>> cmake -DCMAKE_TOOLCHAIN_FILE={path}/armgcc.cmake -G "{MinGW|Unix} Makefiles" -DCMAKE_BUILD_TYPE=debug

        - Start build with make tool or mingw32-make:
        >>> make -C "<path-to-makefile-directory>" -j4
        >>> mingw32-make -C "<path-to-makefile-directory>" -j4

        - Compile. Armgcc compiler will be called to compile in makefile.

    CMake is a cross-platform build system generator. Projects specify their build
    process with platform-independent CMake listfiles included in each directory
    of a source tree with the name CMakeLists.txt. Users build a project by using
    CMake to generate a build system for a native tool on their platform.

    GNU Make is a tool which controls the generation of executables and other non-source
    files of a program from the program's source files. Make gets its knowledge of how to
    build your program from a file called the makefile, which lists each of the non-source
    files and how to compute it from other files. When you write a program, you should
    write a makefile for it, so that it is possible to use Make to build and install the
    program.


    """
    OSLIST = ["Windows","Linux","Darwin"]


    def __init__(self, path, *args, **kwargs):
        super(APP, self).__init__("armgcc", path, *args, **kwargs)


    @property
    def is_ready(self):
        return os.path.exists(self.path)



    @build
    def build_project(self, project, target, logfile, **kwargs):
        """Return a command line string for armgcc. The build commands
        are packaging into a shell/bat script.

        Arguments:
            project {armgcc.Project} -- armgcc project object
            target {string} -- target name
            logfile {string} -- log file path

        Raises:
            IOError -- [if make or cmake path is invalid raise IOError.]

        Returns:
            string -- commandline string.
        """

        osname = platform.system()
        self.path = self.path.replace('\\', '/')
        os.environ["ARMGCC_DIR"] = self.path

        cmake_path = 'cmake'
        make_path = 'make' if osname != 'Windows' else "mingw32-make"

        if "Windows" == osname:
            suffix = 'bat'
        else:
            suffix = 'sh'

        current_path = os.path.dirname(os.path.abspath(__file__))
        script_file = os.path.join(current_path, "_build.{}".format(suffix)).replace('\\', '/')

        buildcmd = "{0} \"{1}\" {2} \"{3}\" \"{4}\" {5} \"{6}\"".format(
            script_file,
            project.prjdir,
            target,
            self._path,
            cmake_path,
            project.armgcc_cmake,
            make_path
        )

        if logfile:
            buildcmd = "{} >> {} 2>&1".format(buildcmd, logfile)

        return buildcmd


    def transform_elf(self, type, in_file, out_file):
        """Convert ELF to specific type.
        Called <mcutk>/bin/arm-none-eabi-objcopy to do the job.

        Supported types: bin, ihex, srec.

        Arguments:
            in_file {str} -- path to elf file.
            out_file {str} -- output file
            type {str} -- which type you want to convert.

        Raises:
            ReadElfError -- Unknown elf format will raise such error
            Exception -- Convert failed will raise exception

        Returns:
            bool
        """
        executor = os.path.join(self.path, 'bin/arm-none-eabi-objcopy')
        if os.name == 'nt':
            executor += '.exe'

        return transform_elf_basic(type, in_file, out_file, executor)



    @staticmethod
    def verify(path):
        '''
        verify the path of compiler is avaliable
        '''
        return os.path.exists(path)



    @staticmethod
    def get_latest():
        """Search and return a latest armgcc instance from system.

        Returns:
            <armgcc.APP object>
        """
        path, version = get_armgcc_latest()
        if path:
            return APP(path, version=version)
        else:
            return None




    @staticmethod
    def parse_build_result(exitcode, logfile):
        """GNU make exits with a status of zero if all makefiles were successfully
        parsed and no targets that were built failed. A status of one will be
        returned if the -q flag was used and  make  determines  that a target
        needs to be rebuilt. A status of two will be returned if any errors
        were encountered.
        """

        if exitcode != 0:
            return BuildResult.map("error")

        if not logfile:
            return BuildResult.map("pass")

        p = re.compile(r'warning:', re.I)

        with open(logfile) as f:
            for line in f:
                if p.search(line) != None:
                    return BuildResult.map("warning")

        return BuildResult.map("pass")




    @staticmethod
    def default_install_path():
        default_path_table = {
            "Windows": "C:/Program Files (x86)/GNU Tools ARM Embedded",
            "Linux": "/usr/local",
            "Darwin": "/usr/local",
        }
        osname = platform.system()
        return default_path_table[osname]




def _scan_windows_register():
    try:
        import _winreg as winreg
    except ImportError:
        import winreg

    root_key_path = r"SOFTWARE\Wow6432Node\ARM\GNU Tools for ARM Embedded Processors"
    try:
        key_object = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, root_key_path)
        path = winreg.QueryValueEx(key_object, "InstallFolder")[0].replace('\\', '/')
        version = path.split("/")[-1].replace(" ", "-")
        winreg.CloseKey(key_object)
        return path, version
    except WindowsError:
        pass

    return None, None




def get_armgcc_latest():
    """Get the latest armgcc version

    Returns:
        tuple -- (path, version)
    """
    path, version = None, None
    osname = platform.system()

    if osname == "Windows":
        path, version = _scan_windows_register()
        if path:
            return path, version

        logging.debug("Could not found ARM GCC installation in windows register!"\
        "Trying to scan in default installation directory!")

    root_path = APP.default_install_path()
    if osname == "Windows" and not os.path.exists(root_path):
        root_path = "C:/Program Files/GNU Tools ARM Embedded"

    return get_armgcc_latest_version(root_path)



def get_armgcc_latest_version(rootpath):
    osname = platform.system()
    versions_dict = {}
    armgcc_rootfolders = []

    if "Windows" == osname:
        armgcc_rootfolders = os.listdir(rootpath)
    else:
        try:
            s = subprocess.check_output("ls {0} | grep gcc-arm-none-eabi".format(rootpath), shell=True)
            armgcc_rootfolders = s.split('\n')
        except subprocess.CalledProcessError:
            return None, None

    for per_folder in armgcc_rootfolders:
        if not os.path.isdir(os.path.join(rootpath, per_folder)) or '' == per_folder:
            continue

        version_content = os.popen("\"{}\" --version".format(os.path.join(rootpath, per_folder, "bin", "arm-none-eabi-gcc"))).read()
        ret = re.search("\d\.\d\.\d", version_content)
        if None != ret:
            versions_dict[ret.group()] = os.path.join(rootpath, per_folder)

    latest_v = sorted(versions_dict.keys(), key=lambda v:LooseVersion(v))[-1]
    return versions_dict[latest_v], latest_v






from __future__ import print_function
import os
import zipfile
import shutil
import logging

from xml.etree import cElementTree as ET
from mcutk.util import rmtree

logger = logging.getLogger(__name__)

def install_pack_unzip(packfile, installdir):
    """
    install cmsis pack by unzip method.
        1. read .pdsc and get 'vendor'/'name'/'releaseversion'
        2. extract all files to installdir
    """
    archive = zipfile.ZipFile(packfile, mode='r')
    files = archive.namelist()
    pdscfilename = [item for item in files if item.endswith(".pdsc")][0]
    content = archive.read(pdscfilename)
    tree = ET.fromstring(content)
    vendor = tree.find("vendor").text
    name = tree.find("name").text
    version = tree.find("releases/release").attrib["version"]

    packdir = os.path.join(installdir, vendor, name)
    if os.path.exists(packdir):
        logging.debug(packfile)
        logger.debug("remove %s", packdir)
        rmtree(packdir)

    install_path = os.path.join(installdir, vendor, name, version)
    logging.debug("  Vendor: {0}, Name: {1}, Version: {2}".format(vendor, name, version))
    # try several times to make sure the folder is removed
    for _ in range(3):
        try:
            if os.path.exists(install_path):
                print("    deleting exists version...")
                rmtree(install_path)
            break
        except Exception as  e:
            print(e)
            print("      failed: try again..")

    os.makedirs(install_path)
    archive.extractall(install_path)
    archive.close()

    if "DFP" in install_path:
        flashalgos = [x.attrib['name'] for x in tree.findall('devices/family/device/algorithm')]
        # validate flash algos
        if flashalgos:
            for algo in flashalgos:
                algo_path = os.path.join(install_path, algo)
                if not os.path.exists(algo_path):
                    logging.critical('Validate FLM failed: Flash Algo "%s" is not exists!', algo_path)
                    return None
        else:
            logging.critical('Validate FLM failed: .pdsc not contains the Flash Algo information!')
            return None



    if os.path.exists(install_path+"/"+pdscfilename):
        logger.debug("install successfully")
        return install_path
    else:
        logger.critical("install failed")
        return None

